<?php
class WordTrailsAnalytics {
    const WEV = "WEV";
    const BEV = "BEV";
    const DIRECT = "direct";
    const INDIRECT = "intersection";
    const CHANCE = "chance";

    const USAGE_INSERT = "INSERT INTO %t (`UsageID`, `Stamp`, `Type`, `TrailHash`, `NodeHash`, `SessionID`, `How`) VALUES (NULL, %s, %s, %s, %s, %s, %s)";
    const PRINT_INSERT = "INSERT INTO %t (`PrintID`,`TrailHash`, `Stamp`, `SessionID`, `ButtonID`, `NodeCount`, `PageCount`, `SectionCount`) VALUES (NULL, %s, %s, %s, %d, %s, %s, %s)";

    const PULL_BY_HASH = "SELECT * FROM %t WHERE `TrailHash` IN (%s)";
    const PULL_BY_SID = "SELECT * FROM %t WHERE `SessionID` = %s";

    const PULL_FOR_TRANSMIT = "SELECT * FROM %t WHERE %s NOT IN (SELECT %s FROM %p)";
    
    const PULL_FOLLOW_COUNTS = "SELECT TrailHash, COUNT(DISTINCT SessionID) as Follows FROM %t WHERE TrailHash IN(%s) GROUP BY TrailHash";

    private static $instance;

    private $hit_list = array();
    private $print_list = array();
    //private static $hit_list = array();
    //private static $print_list = array();
    private static $HIT_DEFAULT = array("TrailHash"=>"", "NodeHash"=>null,"Stamp"=>0, "SessionID"=>"", "Type"=>self::WEV, "How"=>self::CHANCE, "Committed"=>false);
    private static $PRINT_DEFAULT = array("TrailHash"=>"", "Stamp"=>0, "SessionID"=>"", "ButtonID"=>1, "NodeCount"=>0, "PageCount"=>1, "SectionCount"=>1);

    public static $print_buttons = array("Unknown", "WEV", "BEV", "Trail Listing");
    
    public $followcounts = array();

    public function __construct() {
	self::$instance = &$this;
    }
    public function __wakeup() {
	self::$instance = &$this;
    }
    public function __sleep() {
	return array_keys(array_diff(get_object_vars($this), array("followcounts")));
    }
    
    public static function counts($trailhash) {
	if (empty(self::$instance->followcounts)) self::updateFollowCounts();
	if (isset(self::$instance->followcounts[$trailhash])) return (int)self::$instance->followcounts[$trailhash];
	return 0;
    }
    
    private static function updateFollowCounts() {
	global $wpdb;
	$trails = WordTrailsGlobal::getAllTrailHashes();
	$follow_results = $wpdb->get_results(str_replace("%s", implode(",",array_map(array("WordTrailsUtilities", "wrap_with_quotes"),$trails)), str_replace("%t", WordTrailsGlobal::$tables->usage, self::PULL_FOLLOW_COUNTS)), OBJECT);
	self::$instance->followcounts = array();
	foreach ($follow_results as $follow) {
	    self::$instance->followcounts[$follow->TrailHash] = $follow->Follows;
	}
    }

    public static function hit($TrailHash, $NodeHash=null, $Type=self::WEV, $How=self::CHANCE){
	$thisHit = array_merge(self::$HIT_DEFAULT, array("TrailHash"=>$TrailHash, "NodeHash"=>$NodeHash, "Type"=>$Type, "How"=>$How));
	$lastStamp = self::$instance->hit_list[count(self::$instance->hit_list)-1]["Stamp"];
	if (is_null($NodeHash)) {
	    if (false !== self::array_multi_search(array("Stamp"=>$lastStamp, "TrailHash"=>$TrailHash, "Type"=>$Type), self::$instance->hit_list)) return false;
	} else {
	    if (false !== self::array_multi_search(array("Stamp"=>$lastStamp, "NodeHash"=>$NodeHash), self::$instance->hit_list)) return false;
	}
	$thisHit["Stamp"] = date("Y-m-d H:i:s");
	$thisHit["SessionID"] = session_id();
	self::$instance->hit_list[] = $thisHit;
	if ($How == self::DIRECT) self::pushHitsForTrail($TrailHash);
    }

    private static function pushHitsForTrail($TrailHash) {
	$alsoStore = &self::array_multi_search(array("TrailHash"=>$TrailHash, "Committed"=>false), self::$instance->hit_list);
	if (is_array($alsoStore) && !empty($alsoStore)) {
	    foreach ($alsoStore as $id => &$hit) {
		 self::pushHitToDB($hit);
	    }
	}
    }
    private static function pushHitToDB($HitArray) {
	global $wpdb;
	$wpdb->query(
	    str_ireplace(
		array("'null'", '"null"'),
		"NULL",
		$wpdb->prepare(
		    str_replace(
			"%t",
			WordTrailsGlobal::$tables->usage,
			self::USAGE_INSERT
		    ),
		    $HitArray["Stamp"],
		    $HitArray["Type"],
		    $HitArray["TrailHash"],
		    (is_null($HitArray["NodeHash"]) ? "null" : $HitArray["NodeHash"]),
		    $HitArray["SessionID"],
		    $HitArray["How"]
		)
	    )
	);
	$hits = &self::array_multi_search(array("TrailHash"=>$HitArray["TrailHash"], "NodeHash"=>$HitArray["NodeHash"], "Stamp"=>$HitArray["Stamp"]), self::$instance->hit_list);
	foreach ($hits as $key => &$hit) {
	    $hit["Committed"] = true;
	}
    }

    public static function pdf($TrailHash, $ButtonID = null, $NodeCount=null, $PageCount=null, $SectionCount=null) {
	global $wpdb;
	$thisPrint = array_merge(self::$PRINT_DEFAULT, array("TrailHash" => $TrailHash, "ButtonID"=>$ButtonID, "NodeCount"=>$NodeCount, "PageCount"=>$PageCount, "SectionCount"=>$SectionCount));
	$thisPrint["Stamp"] = date("Y-m-d H:i:s");
	$thisPrint["SessionID"] = session_id();
	self::$instance->print_list[] = $thisPrint;
	$wpdb->query(
	    $wpdb->prepare(
		str_replace(
		    "%t",
		    WordTrailsGlobal::$tables->print,
		    self::PRINT_INSERT
		),
		$thisPrint["TrailHash"],
		$thisPrint["Stamp"],
		$thisPrint["SessionID"],
		(int)$thisPrint["ButtonID"],
		$thisPrint["NodeCount"],
		$thisPrint["PageCount"],
		$thisPrint["SectionCount"]
	    )
	);
	self::pushHitsForTrail($TrailHash);
    }

    public static function &array_multi_search($needlestack, &$haystack){
	$results = array();
	foreach ($haystack as $id => &$needle) {
	    if (!is_array($needle)) continue;
	    $match = true;
	    foreach ($needlestack as $key=>$val) {
		if (!array_key_exists($key, $needle)) {
		    $match = false;
		    break;
		}
		if ($needle[$key] !== $val) {
		    $match = false;
		    break;
		}
	    }
	    if ($match) $results[$id] = &$needle;
	}
	return (empty($results) ? false : $results);
    }

    public static function pullAnalyticsFor($hashes = "all", $type = "hash" /* or "sid" */) {
	if (!is_array($hashes)) {
	    if ($type == "hash") {
		if ($hashes == "all") $hashes = WordTrailsGlobal::getAllTrailHashes();
		elseif (WordTrailsUtilities::isHash($hashes))$hashes = array($hashes);
	    } else $hashes = array($hashes);
	}
	if (!is_array($hashes)) return false;
	if ($type == "hash") {
	    $sql = self::PULL_BY_HASH;
	} elseif ($type == "sid") {
	    $sql = self::PULL_BY_SID;
	} else return false;
	global $wpdb;
	$usql = str_replace("%t", WordTrailsGlobal::$tables->usage, $sql);
	$psql = str_replace("%t", WordTrailsGlobal::$tables->print, $sql);
	$uresults = $wpdb->get_results(str_replace("%s", implode(", ", array_map(array("WordTrailsUtilities", "wrap_with_quotes"), $hashes)), $usql));
	$presults = $wpdb->get_results(str_replace("%s", implode(", ", array_map(array("WordTrailsUtilities", "wrap_with_quotes"), $hashes)), $psql));
	$analytics = array();
	foreach ($uresults as $result) {
	    if (!is_array($analytics[$result->TrailHash])) $analytics[$result->TrailHash] = array();
	    if (!is_array($analytics[$result->TrailHash][$result->SessionID])) $analytics[$result->TrailHash][$result->SessionID] = array();
	    if (!is_array($analytics[$result->TrailHash][$result->SessionID]["hit"])) $analytics[$result->TrailHash][$result->SessionID]["hit"] = array();
	    $analytics[$result->TrailHash][$result->SessionID]["hit"]["0".strtotime($result->Stamp)] = $result;
	}
	foreach ($presults as $result) {
	    if (!is_array($analytics[$result->TrailHash])) $analytics[$result->TrailHash] = array();
	    if (!is_array($analytics[$result->TrailHash][$result->SessionID])) $analytics[$result->TrailHash][$result->SessionID] = array();
	    if (!is_array($analytics[$result->TrailHash][$result->SessionID]["print"])) $analytics[$result->TrailHash][$result->SessionID]["print"] = array();
	    $analytics[$result->TrailHash][$result->SessionID]["print"]["0".strtotime($result->Stamp)] = $result;
	}
	foreach ($analytics as $trail => $sessions) {
	    foreach (array_keys($sessions) as $sid) {
		@ksort($analytics[$trail][$sid]["hit"], SORT_NUMERIC);
		@ksort($analytics[$trail][$sid]["print"], SORT_NUMERIC);
	    }
	}
	return $analytics;
    }

    public static function pullAnalyticsToTransmit() {
	global $wpdb;
	$psql = str_replace(array("%t", "%p", "%s"), array(WordTrailsGlobal::$tables->print, WordTrailsGlobal::$tables->print_push, "PrintID"), self::PULL_FOR_TRANSMIT);
	//$psql = str_replace("%s", $psql);
	$usql = str_replace(array("%t", "%t"), array(WordTrailsGlobal::$tables->usage, WordTrailsGlobal::$tables->usage_push), self::PULL_FOR_TRANSMIT);
	$usql = str_replace("%s", "UsageID", $psql);

	$results = array();
	$wpdb->show_errors();
	$presults = $wpdb->get_results($psql);
	$results["print"] = $presults;
	//$uresults = $wpdb->get_results($usql);
	//$results["hit"] = $uresults;
	return $results;
    }

    public static function spitHits() {
	superprint(self::$instance->hit_list, true);
    }
}
?>