<?php
/**
 * This is the Menu class
 *
 * This file contains the WordTrailsMenu class, the one class to rule them all.
 *
 * @package WordTrails
 * @since 0.2.0
 * @author Jesse Silverstein <jesse.silverstein@xerox.com>
 * @copyright 2008 XIG: SemPrint
 * @global WordPress $wpdb WordPress database management class
 */

// {{{ WordTrailsMenu
/**
 * Menu class
 *
 * The menu class installs, displays, and processes all of the WordTrails menus
 *
 * @package WordTrails
 * @since 0.2.0
 * @author Jesse Silverstein <jesse.silverstein@xerox.com>
 * @copyright 2008 XIG: SemPrint
 * @global WordPress $wpdb WordPress database management class
 */
class WordTrailsMenu {
    // {{{ Properties/Variables
    public $topmenu = "Trails";
    protected $menus = array(
	"Edit" => array("func" => "trailsMenu", "slug" => "Edit", "cap" => WordTrailsGlobal::CAP_SAVE)
	,"Add New" => array("func" => "add_new_trail", "slug" => "Add_New", "cap" => WordTrailsGlobal::CAP_SAVE)
	,"Settings" => array("func" => "settingsMenu", "slug" => "Settings", "cap" => 'edit_plugins')
	,"Analytics" => array("func" => "analyticsMenu", "slug" => "Analytics", "cap" => WordTrailsGlobal::CAP_SAVE)
    );
    protected $awake = true;
    protected $modes = array(
	"edit" => "edit_node"
	,"trail_analytics" => "single_trail_analytics"
	,"sid_analytics" => "single_sid_analytics"
    );
    protected $messages = array();
    
    const TRAIL_PAGE_KEY = "trail_page";
    const TRAIL_PAGE_MSG = "You must provide WordTrails with a <a href=\"%s\">page to display Trails</a> for it to work.";
    const TRAIL_PAGE_TITLE = "WordTrails is almost ready.";
    const TRAIL_PAGE_CAP = 'edit_plugins';
    
    const WIDGET_KEY = "WEV_widget";
    const WIDGET_MSG = "You should include the WordTrails Widget in one of your <a href=\"%s\">sidebars</a>";
    const WIDGET_TITLE = "WordTrails is lacking features.";
    const WIDGET_CAP = 'switch_themes';
    
    const TRAILMEME_KEY = "TrailMeme";
    const TRAILMEME_MSG = "Publicize your trails, make them searchable and access advanced features.";
    const TRAILMEME_TITLE = "<a href=\"%s\">Register</a> your plugin with trailmeme.com";
    const TRAILMEME_CAP = 'edit_plugins';

    public $option_num_per_page = array(5, 10, 15, 20, 50);
    // }}}

    public function __sleep() {
	$this->awake = false;
	//$this->messages = array();
	return array_diff(array_keys(get_object_vars($this)), array("menus"));
    }
    public function __wakeup() {
	$this->awake = true;
    }
    public function __toString() {
	return "WordTrails Menu object";
    }
    
    public function __call($name, $arguments) {
	if (isset($this->messages[$name])) {
	    echo "\n";
	    echo $this->messages[$name];
	    echo "\n";
	}
    }

    // {{{ installMenus()
    public function installMenus() {
	$this->continueMenu = true;
	$this->flash_change = array();
	$this->processPOST();
	$page = add_menu_page($this->topmenu, $this->topmenu, $this->menus["Edit"]["cap"], $this->topmenu, array("WordTrailsUtilities", $this->menus["Edit"]["func"]), WordTrailsGlobal::$urlpath . "imgs/logos/tmicon.png");
	add_action("admin_print_scripts-$page", array("WordTrailsUtilities", "menu_head"));
	$first = true;
	$_page = "_page_";
	foreach ($this->menus as $title => $menu) {
	    if ($title == "Analytics") {
		if (0 == WordTrailsGlobal::get_option(WordTrailsGlobal::ANALYTICS_COLLECT_OPTION)) continue;
	    }
	    $page = add_submenu_page($this->topmenu, $this->topmenu . ": " . $title, $title, $menu["cap"], ($first ? $this->topmenu : $menu["slug"]), array("WordTrailsUtilities", $menu["func"]));
	    if ($page) add_action("admin_print_scripts-$page", array("WordTrailsUtilities", "menu_head"));
	    $this->menus[$title]["slug"] = substr($page, strpos($page, $_page) + strlen($_page));
	    $first = false;
	}
	$settings_hook = $this->menus["Settings"]["slug"];
	if (WordTrailsUtilities::warnTrailPage() && current_user_can(self::TRAIL_PAGE_CAP)) {
	    $this->addMessage(sprintf(self::TRAIL_PAGE_MSG, admin_url("admin.php?page={$settings_hook}")), self::TRAIL_PAGE_KEY, self::TRAIL_PAGE_TITLE, true);
	} elseif (WordTrailsUtilities::warnWidget() && current_user_can(self::WIDGET_CAP)) {
	    $this->addMessage(sprintf(self::WIDGET_MSG, admin_url("widgets.php")), self::WIDGET_KEY, self::WIDGET_TITLE, false);
	} elseif (WordTrailsUtilities::warnTrailMeme() && current_user_can(self::TRAILMEME_CAP)) {
	    $this->addMessage(self::TRAILMEME_MSG, self::TRAILMEME_KEY, sprintf(self::TRAILMEME_TITLE, admin_url("admin.php?page={$settings_hook}")), false);
	}

	wp_enqueue_script("wt_menu", WordTrailsGlobal::$urlpath . "js/menu.js", wt_menu_prereqs());
    }

    public function insertCSS() { ?>
<link rel="stylesheet" type="text/css" href="<?php echo WordTrailsGlobal::$urlpath; ?>css/menu.css" />
<?php
    }
    // }}}

    public function printMessages() {
	return false;
	if (count($this->messages) > 0) {
	    foreach ($this->messages as $message) {
		echo "<br style=\"clear:both;\" /><div id=\"message\" class=\"updated fade\"><p>$message</p></div>";
	    }
	}
    }
    
    public function addMessage($message, $key, $title = "", $error = false) {
	$before = "<div id=\"message-{$key}\" class=\"" . ($error ? "error" : "updated fade") . "\"><p>";
	$after = "</p></div>";
	if (!empty($title)) $title = "<strong>{$title}</strong> ";
	$this->messages[$key] = $before . $title . $message . $after;
	add_action("admin_notices", array($this, $key));
    }


    public function processPOST() {
	if (!isset($_POST['process']) || ! is_array($_POST['process'])) return 0;
	$ret = array();
	//superdump($_POST, true);
	foreach ($_POST['process'] as $process) {
	    if (!is_array($process) || empty($process['func']) || empty($process['args'])) continue;
	    $ret[] = $this->$process['func']($process['args']);
	    if (!$this->continueMenu) break;
	}
	return $ret;
    }

    public function href_to_edit_node($hash, $vars = array()) {
	$rtn =  "?page=" . $this->topmenu . "&amp;mode=edit&amp;hash=$hash";
	$vars = array_diff($vars, array("page", "edit", "hash"));
	foreach ($vars as $var) {
	    if (isset($_GET[$var]))
		$rtn .= "&amp;" . $var . "=" . $_GET[$var];
	}
	return $rtn;
    }
    public function href_to_submit($hash = null) {
	if ($_GET["page"] == $this->menus["Add New"]["slug"] && !is_null($hash)) return str_replace("&amp;", "&", $this->href_to_edit_node($hash));
	return str_replace( '%7E', '~', $_SERVER['REQUEST_URI']);
    }
    public function build_href($newvars = array(), $trash_vars = array()) {
	$allvars = array_diff(array_unique(array_merge(array_keys($_GET), array_keys((array)$newvars))), array("page", "reverse"), (array)$trash_vars);
	$href = "?page=";
	if (isset($newvars["page"])) $href .= $newvars["page"];
	else $href .= $_GET["page"];
	foreach ($allvars as $var) {
	    if (isset($newvars[$var])) $href .= "&amp;{$var}={$newvars[$var]}";
	    elseif (isset($_GET[$var]) && !empty($_GET[$var])) $href .= "&amp;{$var}={$_GET[$var]}";
	}
	return $href;
    }

    public function trailsMenu() {
	//$this->continueMenu = true;
	global $wpdb;
	//$this->processPOST();
	if (isset($_GET["mode"])) {
	    if (false !== array_search($_GET["mode"], array_keys($this->modes))) {
		call_user_func(array($this, $this->modes[$_GET["mode"]]));
	    }
	}
	if (!$this->continueMenu) return;
	$trails = WordTrailsGlobal::getAllTrailNodes("Name, ShortDesc");
	$tpp = "trails_per_page";
	$tp = "trails_paged";
	$total = (count($trails["db"])+count($trails["live"]));
	$maxcount = (int)$_GET[$tpp]>0 ? (int)$_GET[$tpp]:10;
	$page = (int)$_GET[$tp]>0 ? (int)$_GET[$tp]:1;
	$start = ($page-1)*$maxcount;
	while ($start > $total) {
	    $start -= $maxcount;
	    $page--;
	}
	$page_links = paginate_links( array(
	    'base' => add_query_arg( $tp, '%#%' ),
	    'format' => '',
	    'total' => ceil($total/$maxcount),
	    'current' => $page
	));

	?>

<div class="wrap">
    <br class="clear" />
    <h2><?php _e( 'Edit Trails', WordTrailsGlobal::DOMAIN ); ?></h2><?php
	//$this->printMessages();
	$this->num_per_page($tpp, 10, $tp);
	if ($page_links) { ?>
    <br class="clear" />
    <div class="tablenav">
	<div class='tablenav-pages'><?php echo $page_links; ?></div>
    </div><?php
	} ?>

    <br class="clear" />
    <table class="widefat" cellspacing="4">
    <thead>
    <tr>
        <th scope="col"><?php _e( 'Trail Name', WordTrailsGlobal::DOMAIN ); ?></th>
        <th scope="col"><?php _e( 'Caption', WordTrailsGlobal::DOMAIN ); ?></th>
        <th scope="col" class="num"><?php _e( 'Number of Markers', WordTrailsGlobal::DOMAIN ); ?></th>
        <th scope="col" class="num"><?php _e( 'Unsaved Changes', WordTrailsGlobal::DOMAIN ); ?></th>
        <th scope="col"><?php _e( 'Actions' ); ?></th>
    </tr>
    </thead>
    <tbody>
    <?php
    if (is_array($trails) && (!empty($trails["db"]) || !empty($trails["live"]))) {
	$alt = false;
	$count = 0;
	foreach ($trails["db"] as $trail) {
	    $count++;
	    if ($count <= $start) continue;
	    if ($count > ($maxcount+$start)) break;
	    echo "\n\t<tr";
            if ( $alt ) echo ' class="alternate"';
	    echo ">";
	    $alt = !$alt;
	    echo "\n\t\t<td align=\"left\">" . $trail->Name . "</td>";
	    echo "\n\t\t<td align=\"left\">" . $trail->ShortDesc . "</td>";
	    $num = (int)$wpdb->get_var($wpdb->prepare("SELECT COUNT(ChildID) FROM " . WordTrailsGlobal::$tables->node_rel . " WHERE ParentID = %d GROUP BY ParentID", $trail->NodeID));
	    ?>

	<td class="num"><?php echo $num; ?></td>
	<td class="num">0</td>
	<td align="left">
	    <form name="modify" id="form_modify_<?php echo $trail->NodeID; ?>" method="post" action="<?php echo $this->href_to_submit(); ?>">
	    <input id="form_modify_func_<?php echo $trail->NodeID; ?>" type="hidden" name="process[modify][func]" value="" />
	    <input id="form_modify_arg_hash_<?php echo $trail->NodeID; ?>" type="hidden" name="process[modify][args][hash]" value="<?php echo $trail->Hash; ?>" />
	    <input type="button" class="<?php wt_button_class(); ?>"  value="Edit" onclick="jQuery('#form_modify_func_<?php echo $trail->NodeID; ?>').attr('value','to_edit_node');jQuery('#form_modify_<?php echo $trail->NodeID; ?>').submit();" />
	    <input type="button" class="<?php wt_button_class(); ?> trail-delete"  value="Delete";" />
	    </form>
	</td>
    </tr><?php
	}
	/*jQuery('#form_modify_func_<?php echo $trail->NodeID; ?>').attr('value','delete_node');jQuery('#form_modify_<?php echo $trail->NodeID; ?>').submit();*/
	foreach ($trails["live"] as $trail) {
	    $count++;
	    if ($count <= $start) continue;
	    if ($count > ($maxcount+$start)) break;
	    echo "\n\t<tr";
            if ( $alt ) echo ' class="alternate"';
	    echo ">";
	    $alt = !$alt;
	    echo "\n\t\t<td align=\"left\">" . $trail->getName() . "</td>";
	    echo "\n\t\t<td align=\"left\">" . $trail->getShortDesc() . "</td>";
	    //$num = max(0,(int)$wpdb->get_var($wpdb->prepare("SELECT COUNT(ChildID) FROM " . WordTrailsGlobal::$tables->node_rel . " WHERE ParentID = %d GROUP BY ParentID", $trail->getNodeID())));
	    echo "\n\t\t<td class=\"num\">" . count($trail->child_hashes) . "</td>";
	    echo "\n\t\t<td class=\"num\">" . count($trail->unsaved()) . "</td>";?>

	<td align="left">
	    <form name="modify" id="form_modify_<?php echo $trail->getNodeID(); ?>" method="post" action="<?php echo $this->href_to_submit(); ?>">
	    <input id="form_modify_func_<?php echo $trail->getNodeID(); ?>" type="hidden" name="process[modify][func]" value="" />
	    <input id="form_modify_arg_hash_<?php echo $trail->getNodeID(); ?>" type="hidden" name="process[modify][args][hash]" value="<?php echo $trail->getHash(); ?>" />
	    <input type="button" class="<?php wt_button_class(); ?>"  value="Edit" onclick="jQuery('#form_modify_func_<?php echo $trail->getNodeID(); ?>').attr('value','to_edit_node');jQuery('#form_modify_<?php echo $trail->getNodeID(); ?>').submit();" />
	    <input type="button" class="<?php wt_button_class(); ?> trail-delete"  value="Delete" />
	    <?php if ($trail->hasUnsaved()) { ?>

	    <input type="button" class="<?php wt_button_class(); ?>"  value="Save" onclick="jQuery('#form_modify_func_<?php echo $trail->getNodeID(); ?>').attr('value','save_node');jQuery('#form_modify_<?php echo $trail->getNodeID(); ?>').submit();" /><?php } ?>
	    </form>
	</td><?php
	    echo "\n\t</tr>";
	}
    } else {
	echo "\n\t<tr><td colspan=\"5\" align=\"center\"><i>No Trails Found</i></td></tr>";
    }
    ?>

    </form>
    </tbody>
    </table><?php 	if ($page_links) { ?>
    <br class="clear" />
    <div class="tablenav">
	<div class='tablenav-pages'><?php echo $page_links; ?></div>
    </div><?php
	} ?>
<br /><br />
</div>
    <?php
    }
    
    public function add_new_trail() {
	$trail = WordTrailsGlobal::createNewNode(true, "trail");
	if (is_object($trail)) {
	    $this->edit_node($trail->getHash());
	}
    }

    public function add_trail($args) {
	if (!is_array($args)) return false;
	extract($args);
	$trail = WordTrailsGlobal::createNewNode(true, "trail");
	if (is_object($trail)) {
	    $trail->setName($trail_name);
	    $trail->setShort($trail_desc);
	    $this->to_edit_node(array("hash"=>$trail->getHash()));
	}
    }
    public function save_node($args) {
	extract($args);
	if (!isset($hash)) return false;
	global $wpdb;
	$node = WordTrailsGlobal::getNode($hash);
	if (!is_object($node)) return false;
	$wpdb->show_errors();
	$node->save();
    }
    public function to_edit_node($args) {
	extract($args);
	if (!isset($hash)) return false;
	header("Location: " . str_replace("&amp;", "&", $this->href_to_edit_node($hash)));
    }
    public function edit_node($hash = null) {
	if (isset($_GET["hash"])) {
	    $hash = $_GET["hash"];
	} elseif ($_GET["page"] != $this->menus["Add New"]["slug"]) {
	    return false;
	}
	if (is_null($hash)) return false;
	global $wpdb;
	$node = WordTrailsGlobal::getNode($hash);
	if (!is_object($node)) return false;
	$this->continueMenu = false;
	$type = $node->getType();
	$child_type = "child";
	$children_type = "children";
	if ($type == "trail") {
	    $child_type = "marker";
	    $children_type = "markers";
	}

	add_meta_box("editnodeinfo", ucwords($type) . " Info", array($this, "edit_node_info_form"), "editnode");
	add_meta_box("editnodechildren", "Manage " . ucwords($children_type), array($this, "edit_node_children_form"), "editnode");
	if (!empty($node->trail_hash) && $node->trail_hash != $hash) {
	    //error_log("I am: $type ($hash)");
	    //error_log("My Trail Hash: " . $node->trail_hash);
	    //add_meta_box("editnodechildren", "Manage " . ucwords($children_type), array($this, "edit_node_children_form"), "editnode");
	    $trail = WordTrailsGlobal::getNode($node->trail_hash);
	    if (is_object($trail)) {
		add_meta_box("editnodeaddchildren", "Add " . ucwords($children_type) . " from \"" . $trail->getName() . "\" Trail", array($this, "edit_node_add_children_form"), "editnode");
	    }
	} else {
	    //add_meta_box("editvisualtrail", "Visually Edit Trail", array($this, "edit_visual_trail"), "editnode");
	    add_meta_box("editnodeaddpost", "Add " . ucwords($children_type) . " From Posts on This Blog", array($this, "edit_node_add_post_form"), "editnode");
	    add_meta_box("editnodeaddurl", "Create " . ucwords($child_type) . " From an External URL", array($this, "edit_node_add_url_form"), "editnode");
	}
	//add_meta_box("editnodeaddexttrail", "Create " . ucwords($children_type) . " From Another Plugin-enabled Blog", array($this, "edit_node_add_p2p_form"), "editnode");

	?>
<div class="wrap">
<br style="clear: both;" />
<h2>
    Editing <?php echo ucwords($type); ?>: <a href="<?php echo $this->href_to_edit_node($node->getHash()); ?>"><?php
$nname = $node->getName();
if (empty($nname)) $nname = "<i>Unnamed</i>";
echo "{$nname}</a>";
?>
    <a href="<?php echo WordTrailsUtilities::redirect_link_to("xml", $hash); ?>" class="ltoh">
	<img src="<?php echo WordTrailsGlobal::$urlpath; ?>imgs/buttons/xml_l.png" alt="View XML" title="View XML" />
    </a>
</h2>
<?php
$unsaved = false;
if ($type == "node" && !is_null($node->trail_hash)) {
    $trail = WordTrailsGlobal::getNode($node->trail_hash);
    if (is_object($trail)) {
	$tname = $trail->getName();
	if ($trail->hasUnsaved()) $unsaved = true;
	if (empty($tname)) $tname = "<i>Unnamed</i>"; ?>

<br style="clear: both;" />
<h2 style="border-bottom: none;margin:0px;padding:0px;float:left;">
    <small>
	on Trail: <a href="<?php echo $this->href_to_edit_node($trail->getHash()); ?>"><?php echo $tname; ?></a>
	<a href="<?php echo WordTrailsUtilities::redirect_link_to("xml", $trail->getHash()); ?>" class="ltoh">
	    <img src="<?php echo WordTrailsGlobal::$urlpath; ?>imgs/buttons/xml_l.png" alt="View Trail XML" title="View TrailXML" />
	</a>
    </small>
</h2><?php
    }
} else if ($node->hasUnsaved()) {
    $trail = &$node;
    $unsaved = true;
}
if ($unsaved) {
    ?>

<form style="float:right;" name="save_trail" method="post" action="<?php echo $this->href_to_submit($trail->getHash()); ?>">
    <h4 class="updated flash-change">
	(<?php echo count($trail->unsaved());
?> Trail Change<?php echo count($trail->unsaved()) != 1 ? "s" : "";?> &amp; <?php $usc = $trail->countUnsavedChildren(); echo $usc; ?> Unsaved Marker<?php echo $usc != 1 ? "s" : "";
?>)
	    <input type="submit" class="<?php wt_button_class(); ?>" style="font-size:8pt;padding:1px;"  value="Commit to Database" />
    </h4>
    <input type="hidden" name="process[save][func]" value="save_node" />
    <input type="hidden" name="process[save][args][hash]" value="<?php echo $trail->getHash(); ?>" />
</form><?php
}
//$this->printMessages();
?>
<br style="clear: both;" />
<div id="poststuff">

<?php
do_meta_boxes("editnode", 'advanced', $hash);
wp_nonce_field( 'closedpostboxes', 'closedpostboxesnonce', false );
wp_nonce_field( 'meta-box-order', 'meta-box-order-nonce', false );
?>

</div>
</div>
<script language="JavaScript" type="text/javascript">
<!--

    var ajax_url = "<?php bloginfo( 'wpurl' ); ?>/wp-admin/admin-ajax.php";

    <?php post_box_toggles("editnode"); ?>

    // close postboxes that should be closed
    jQuery('.if-js-closed').removeClass('if-js-closed').addClass('closed');

    // show things that should be visible, hide what should be hidden
    jQuery('.hide-if-no-js').show();
    jQuery('.hide-if-js').hide();

//-->
</script>
<?php
    }

    public function edit_node_info_form($hash) {
	$node = WordTrailsGlobal::getNode($hash);
	if (!is_object($node)) return false;
	$jumphash = "#node_info"; ?>

<a name="<?php echo substr($jumphash,1); ?>"></a>
<div style="float:left" id="TrailViewer">
<form name="edit_node_info" method="post" action="<?php echo $this->href_to_submit($hash); ?>" style="display: inline;"><?php
	if ($node->delete_on_save) { ?>
<input type="hidden" name="process[info][func]" value="un_delete_node" />
<input type="hidden" name="process[info][args][hash]" value="<?php echo $node->getHash(); ?>" />
<?php $node->editInfoForm(); ?>
<input type="submit" class="<?php wt_button_class(); ?>" value="Restore" /><?php
	} else { ?>
<input type="hidden" name="process[info][func]" value="edit_node_info" />
<input type="hidden" name="process[info][args][hash]" value="<?php echo $node->getHash(); ?>" />
<?php $node->editInfoForm(); ?>
<input type="submit" class="<?php wt_button_class(); ?>" value="Update" />
</form><form name="delete_node" method="post" action="<?php echo $this->href_to_submit($hash); ?>" style="display: inline;">
<input type="hidden" name="process[info][func]" value="delete_node" />
<input type="hidden" name="process[info][args][hash]" value="<?php echo $node->getHash(); ?>" />
<input type="submit" class="<?php wt_button_class(); ?>" value="Delete" /><?php
	} ?>
</form>
</div><?php
	if (strtolower($node->getType()) == "trail") {
	    $session = WordTrailsAnalytics::pullAnalyticsFor(session_id(), "sid");
	    $follows = $session[$node->getHash()][session_id()]["hit"];
	    if (!empty($follows)) {
		    $follows = array_filter($follows, array("WordTrailsUtilities", "onlyWEV"));
		    if (!empty($follows)) {
			    global $wp_wt_sort;
			    $old_sort = $wp_wt_sort;
			    $wp_wt_sort = "Stamp";
			    uasort($follows, array("WordTrailsUtilities", "array_object_sort"));
			    $wp_wt_sort = $old_sort;
			    //$lastFollowed = end($follows);
			    //reset($follows);
			    $follows = array_map(array("WordTrailsUtilities", "extractNodeHash"), array_values($follows));
		    }
	    }
	    $xml = WordTrailsUtilities::redirect_link_to("xml", $node->getHash());
	    $flashvars = array();
	    if (!empty($follows)) $flashvars["follows"] = str_replace('"', "'", json_encode($follows));
	    $flashvars["save_uri"] = urlencode(WordTrailsUtilities::redirect_link_to("tv_save", 1));
	    $flashvars["save_xml_var"] = "xml";
	    $flashvars["redirect_after_save"] = urlencode("http://".$_SERVER['SERVER_NAME'].$_SERVER["SCRIPT_NAME"].$this->href_to_submit($hash));
	    $override_backup_div = true;
	    include "visual_trail_wide.inc";
	}
	?>
<br class="clear" /><?php
    }

    public function edit_node_info($args) {
	extract($args);
	if (!isset($hash)) return false;
	$node = WordTrailsGlobal::getNode($hash);
	if (!is_object($node)) return false;
	$node->editInfoFormProcess($args);
    }
    public function edit_node_add_children_form($hash) {
	global $wpdb;
	$node = WordTrailsGlobal::getNode($hash);
	if (!is_object($node) || empty($node->trail_hash)) return false;
	$trail = WordTrailsGlobal::getNode($node->trail_hash);
	if (!is_object($trail) || empty($trail->child_hashes)) return false;
	$jumphash = "#add_children";
	$type = $node->getType();
	$child_type = "child";
	$children_type = "children";
	if ($type == "trail") {
	    $child_type = "marker";
	    $children_type = "markers";
	}
	list($trail_live, $trail_db) = $trail->getLiveChildrenAndHashes();
	$trail_live = array_diff($trail_live, array($hash), $node->child_hashes);
	$trail_db = array_diff($trail_db, array($hash), $node->child_hashes);

	$cpp = "siblings_per_page";
	$cp = "sibling_paged";
	$total = (count($trail_live)+count($trail_db));
	$maxcount = (int)$_GET[$cpp]>0 ? (int)$_GET[$cpp]:10;
	$page = (int)$_GET[$cp]>0 ? (int)$_GET[$cp]:1;
	$start = ($page-1)*$maxcount;
	while ($start > $total) {
	    $start -= $maxcount;
	    $page--;
	}
	$count = 0;
	$page_links = paginate_links( array(
	    'base' => add_query_arg( $cp, '%#%' ) . $jumphash,
	    'format' => '',
	    'total' => ceil($total/$maxcount),
	    'current' => $page
	));
	?>
<a name="<?php echo substr($jumphash, 1); ?>"></a>
<?php $this->num_per_page($cpp, 10, $cp, $jumphash); ?>
<form name="edit_node_add_children" id="children_add_existing" method="post" action="<?php echo $this->href_to_submit($hash) . "#edit_children"; ?>">
<input type="hidden" id="existing_children_func" name="process[add_children][func]" value="" />
<input type="hidden" name="process[add_children][args][parenthash]" value="<?php echo $hash; ?>" />
<div class="tablenav">
    <?php if ($page_links) echo "<div class='tablenav-pages'>{$page_links}</div>"; ?>
    <input type="submit" class="<?php wt_button_class(); ?>" onclick="jQuery('#existing_children_func').attr('value', 'add_existing_children');" value="Add Children" />
</div>
<br class="clear" />
    <table class="widefat" cellspacing="4">
    <thead>
    <tr valign="top" class="thead">
	<th scope="col" class="check-column"><input type="checkbox"<?php wt_checkAll("document.getElementById('children_add_existing')");?> /></th>
        <th scope="col"><?php _e( 'Marker Name', WordTrailsGlobal::DOMAIN ); ?></th>
        <th scope="col"><?php _e( 'Caption', WordTrailsGlobal::DOMAIN ); ?></th>
        <th scope="col" class="num"><?php _e( 'Number of Children', WordTrailsGlobal::DOMAIN ); ?></th>
        <th scope="col" class="num"><?php _e( 'Unsaved Changes', WordTrailsGlobal::DOMAIN ); ?></th>
        <th scope="col" width="60px"><?php _e( 'Action(s)' ); ?></th>
    </tr>
    </thead>
    <tbody>
    <?php
    if (!empty($trail_db) || !empty($trail_live)) {
	$alternate = true;
	foreach ($trail_live as $hash) {
	    $alternate = !$alternate;
	    $child = WordTrailsGlobal::getNode($hash);
	    if (!is_object($child)) continue;
	    $count++;
	    if ($count <= $start || $count > $start+$maxcount) continue; ?>

    <tr class="<?php echo in_array($hash, $this->flash_change) ?"flash-change":""; echo $alternate ? " alternate":""; ?>">
	<th scope="row" class="check-column"><input type="checkbox" name="process[add_children][args][children][]" value="<?php echo $hash; ?>" /></th>
	<td><?php echo $child->getName(); ?></td>
	<td><?php echo $child->getShortDesc(); ?></td>
	<td class="num"><?php echo count($child->child_hashes); ?></td>
	<td class="num"><?php echo count($child->unsaved()); ?></td>
	<td>
	    <input id="form_modify_func_<?php echo $child->getNodeID(); ?>" type="hidden" name="process[modify_<?php echo $child->getNodeID(); ?>][func]" value="" />
	    <input id="form_modify_arg_hash_<?php echo $child->getNodeID(); ?>" type="hidden" name="process[modify_<?php echo $child->getNodeID(); ?>][args][hash]" value="<?php echo $child->getHash(); ?>" />
	    <a href="<?php echo $this->href_to_edit_node($child->getHash()); ?>" class="ltoh"><img src="<?php echo WordTrailsGlobal::$urlpath; ?>imgs/buttons/edit_l.png" alt="Edit" /></a><?php
	    if ($child->delete_on_save) { ?>

	    <a href="#" class="ltoh" onclick="jQuery('#form_modify_func_<?php echo $child->getNodeID(); ?>').attr('value','un_delete_node');jQuery(this).parents('form').submit();"><img src="<?php echo WordTrailsGlobal::$urlpath; ?>imgs/buttons/undelete_l.png" alt="Un-Delete" /></a><?php
	    } else if ($type == "trail") { ?>

	    <a href="#" class="ltoh" onclick="jQuery('#form_modify_func_<?php echo $child->getNodeID(); ?>').attr('value','delete_node');jQuery(this).parents('form').submit();"><img src="<?php echo WordTrailsGlobal::$urlpath; ?>imgs/buttons/delete_l.png" alt="Delete" /></a><?php
	    } ?>

	</td>
    </tr><?php
	}
	if (!empty($trail_db)) {
	    $sql = "SELECT n.NodeID, n.Hash, n.Name, n.ShortDesc FROM " . WordTrailsGlobal::$tables->node . " as n WHERE n.Hash IN (" .
		implode(",",array_map(array("WordTrailsUtilities", "wrap_with_quotes"), $trail_db)) .
		") ORDER BY n.NodeID DESC";
	    $children_from_db = $wpdb->get_results($sql, OBJECT);
	    $countsql = "SELECT COUNT(nr.ChildID) FROM " . WordTrailsGlobal::$tables->node_rel . " as nr WHERE nr.ParentID = %d GROUP BY nr.ParentID";
	    if (is_array($children_from_db)) {
		foreach ($children_from_db as $child) {
		    $count++;
		    $alternate = !$alternate;
		    if ($count <= $start || $count > $start+$maxcount) continue;
		    $numchildren = $wpdb->get_var($wpdb->prepare($countsql, $child->NodeID));
		    $numchildren = (int)$numchildren;?>
    <tr class="<?php echo in_array($child->Hash, $this->flash_change) ?"flash-change":""; echo $alternate ? " alternate":""; ?>">
	<th scope="row" class="check-column"><input type="checkbox" name="process[add_children][args][children][]" value="<?php echo $child->Hash; ?>" /></th>
	<td><?php echo $child->Name; ?></td>
	<td><?php echo $child->ShortDesc; ?></td>
	<td class="num"><?php echo $numchildren; ?></td>
	<td class="num">0</td>
	<td>
	    <input id="form_modify_func_<?php echo $child->NodeID; ?>" type="hidden" name="process[modify_<?php echo $child->NodeID; ?>][func]" value="" />
	    <input id="form_modify_arg_hash_<?php echo $child->NodeID; ?>" type="hidden" name="process[modify_<?php echo $child->NodeID; ?>][args][hash]" value="<?php echo $child->Hash; ?>" />
	    <a href="<?php echo $this->href_to_edit_node($child->Hash); ?>" class="ltoh"><img src="<?php echo WordTrailsGlobal::$urlpath; ?>imgs/buttons/edit_l.png" alt="Edit" /></a><?php
	     if ($type == "trail") { ?>

	    <a href="#" class="ltoh" onclick="jQuery('#form_modify_func_<?php echo $child->NodeID; ?>').attr('value','delete_node');jQuery(this).parents('form').submit();"><img src="<?php echo WordTrailsGlobal::$urlpath; ?>imgs/buttons/delete_l.png" alt="Delete" /></a><?php
	     } ?>

	</td>
    </tr><?php
		}
	    }
	}
    } else { ?>

    <tr><td colspan="6" align="center"><i>No Other Markers Found</i></td></tr><?php
    }

    ?>
    </tbody>
</table>
<div class="tablenav">
    <?php if ($page_links) echo "<div class='tablenav-pages'>{$page_links}</div>"; ?>
    <input type="submit" class="<?php wt_button_class(); ?>" onclick="jQuery('#existing_children_func').attr('value', 'add_existing_children');" value="Add Children" />
</div>
<br class="clear" />
</form><?php
    }

    public function add_existing_children($args) {
	extract($args);
	if (!is_array($children) || empty($children)) return false;
	$parent = WordTrailsGlobal::getNode($parenthash);
	if (!is_object($parent)) return false;
	$parent->addChild($children);
	$this->addMessage("Added " . count($children) . " Child". (count($children) != 1 ? "ren" : "") . " to this node", "added_children_notice");
	$this->flash_change += $children;
	/*
	?><div id="message" class="updated fade">Added <?php echo count($children); ?> Child<?php echo count($children) != 1 ? "ren" : "";?> to this node</div><?php
	*/
    }

    public function delete_relationships($args) {
	extract($args);
	if (!is_array($children) || empty($children)) return false;
	$parent = WordTrailsGlobal::getNode($parenthash);
	if (!is_object($parent)) return false;
	$parent->removeChild($children);
	$this->addMessage("Removed " . count($children) . " Child" . (count($children) != 1 ? "ren" : "") . " from this node", "removed_childre_notice");
	$this->flash_change += $children;
	/*
	?><div id="message" class="updated fade">Removed <?php echo count($children); ?> Child<?php echo count($children) != 1 ? "ren" : "";?> from this node</div><?php
	*/
    }

    public function num_per_page($var, $default_npp = 10, $vars_to_trash = null, $jumphash = "") {
	?><div class="alignright" style="font-size: 12px;"><ul class="subsubsub" style="display:inline;"><?php
	$size_links = array();
	if (0 == (int)$_GET[$var]) $_GET[$var] = $default_npp;
	foreach ($this->option_num_per_page as $size) {
	    if ((int)$_GET[$var] == $size)
		$size_links[] = "<li><a href='javascript:void(0)' class='current'>{$size}</a>";
	    else
	        $size_links[] = "<li><a href='{$this->build_href(array("$var" => $size), $vars_to_trash)}{$jumphash}' {$class}>{$size}</a>";
	}
	echo implode(' |</li>', $size_links) . '</li>';
	unset($size_links);
	if ($_GET[$var] == $default_npp)
	    unset($_GET[$var]);
	?></ul></div><br class="clear" /><?php
    }

    public function edit_node_children_form($hash) {
	global $wpdb;
	$node = WordTrailsGlobal::getNode($hash);
	if (!is_object($node)) return false;
	$jumphash = "#edit_children";
	$type = $node->getType();
	$child_type = "child";
	$children_type = "children";
	if ($type == "trail") {
	    $child_type = "marker";
	    $children_type = "markers";
	}
	$cpp = "children_per_page";
	$cp = "child_paged";
	$total = count($node->child_hashes);
	$maxcount = (int)$_GET[$cpp]>0 ? (int)$_GET[$cpp]:10;
	$page = (int)$_GET[$cp]>0 ? (int)$_GET[$cp]:1;
	$start = ($page-1)*$maxcount;
	while ($start > $total) {
	    $start -= $maxcount;
	    $page--;
	}
	$count = 0;
	$page_links = paginate_links( array(
	    'base' => add_query_arg( $cp, '%#%' ) . $jumphash,
	    'format' => '',
	    'total' => ceil($total/$maxcount),
	    'current' => $page
	));
	?>
<a name="<?php echo substr($jumphash,1); ?>"></a>
<?php $this->num_per_page($cpp, 10, $cp, $jumphash); ?>
<form name="edit_node_del_children" id="children_del_rel" method="post" action="<?php echo $this->href_to_submit($hash) . $jumphash; ?>">
<input type="hidden" id="remove_children_func" name="process[del_rel][func]" value="" />
<input type="hidden" name="process[del_rel][args][parenthash]" value="<?php echo $hash; ?>" />
<?php if ($page_links || $type != "trail") { ?><div class="tablenav">
    <?php if ($page_links) echo "<div class='tablenav-pages'>{$page_links}</div>"; ?>
    <?php if ($type != "trail") { ?><input type="submit" class="<?php wt_button_class(); ?>" onclick="jQuery('#remove_children_func').attr('value', 'delete_relationships');" value="Delete Relationships" /><?php } ?>

</div><?php } ?>

<br class="clear" />
    <table class="widefat" cellspacing="4">
    <thead>
    <tr>
	<th scope="col" class="check-column"><input type="checkbox" <?php if ($type == "trail") { ?>disabled="disabled"<?php } else { wt_checkAll("document.getElementById('children_del_rel')"); } ?> /></th>
        <th scope="col"><?php _e( ucwords($child_type) . ' Name', WordTrailsGlobal::DOMAIN ); ?></th>
        <th scope="col"><?php _e( 'Caption', WordTrailsGlobal::DOMAIN ); ?></th>
        <th scope="col" class="num"><?php _e( 'Number of Children', WordTrailsGlobal::DOMAIN ); ?></th>
        <th scope="col" class="num"><?php _e( 'Default', WordTrailsGlobal::DOMAIN ); ?></th>
        <th scope="col" class="num"><?php _e( 'Unsaved Changes', WordTrailsGlobal::DOMAIN ); ?></th>
        <th scope="col" width="90px"><?php _e( 'Action(s)' ); ?></th>
    </tr>
    </thead>
    <tbody>
    <?php
    if ($node->hasChildren()) {
	list($live_children, $hashes_from_db) = $node->getLiveChildrenAndHashes();
	$alternate = true;
	foreach ($live_children as $hash) {
	    $alternate = !$alternate;
	    $child = WordTrailsGlobal::getNode($hash);
	    if (!is_object($child)) continue;
	    $count++;
	    if ($count <= $start) continue;
	    if ($count > $start+$maxcount) break; ?>

    <tr class="<?php echo in_array($hash, $this->flash_change) ?"flash-change":""; echo $alternate ? " alternate":""; ?>">
	<th scope="row" class="check-column"><input type="checkbox" name="process[del_rel][args][children][]" value="<?php echo $child->getHash(); ?>" <?php echo ($type == "trail" && $child->getType() != "trail") ? "disabled=\"disabled\" " : ""; ?>/></th>
	<td><?php echo $child->getName(); ?></td>
	<td><?php echo $child->getShortDesc(); ?></td>
	<td class="num"><?php echo count($child->child_hashes); ?></td>
	<td class="num"><input type="radio" name="child_radio_<?php echo $hash; ?>" onclick="setDefault('<?php echo $node->getHash(); ?>','<?php echo $hash; ?>', '<?php echo $jumphash; ?>');return false;" <?php echo ($node->isDefault($hash) ? "checked=\"checked\"" : ""); ?> /></td>
	<td class="num"><?php echo count($child->unsaved()); ?></td>
	<td>
	    <input id="form_modify_func_<?php echo $child->getNodeID(); ?>" type="hidden" name="process[modify_<?php echo $child->getNodeID(); ?>][func]" value="" />
	    <input id="form_modify_arg_hash_<?php echo $child->getNodeID(); ?>" type="hidden" name="process[modify_<?php echo $child->getNodeID(); ?>][args][hash]" value="<?php echo $child->getHash(); ?>" />
	    <a href="<?php echo $this->href_to_edit_node($child->getHash()); ?>" class="ltoh"><img src="<?php echo WordTrailsGlobal::$urlpath; ?>imgs/buttons/edit_l.png" alt="Edit" /></a><?php
	    if ($child->delete_on_save) { ?>

	    <a href="#" class="ltoh" onclick="jQuery('#form_modify_func_<?php echo $child->getNodeID(); ?>').attr('value','un_delete_node');jQuery(this).parents('form').submit();"><img src="<?php echo WordTrailsGlobal::$urlpath; ?>imgs/buttons/undelete_l.png" alt="Un-Delete"/></a><?php
	    } else if ($type == "trail") { ?>

	    <a href="#" class="ltoh" onclick="jQuery('#form_modify_func_<?php echo $child->getNodeID(); ?>').attr('value','delete_node');jQuery(this).parents('form').submit();"><img src="<?php echo WordTrailsGlobal::$urlpath; ?>imgs/buttons/delete_l.png" alt="Delete"/></a><?php
	    } ?>

	</td>
    </tr><?php
	}
	if (!empty($hashes_from_db)) {
	    $sql = "SELECT n.NodeID, n.Hash, n.Name, n.ShortDesc, n.Type FROM " . WordTrailsGlobal::$tables->node . " as n ";
	    $sql .= "WHERE n.Hash IN (" .
		implode(",",array_map(array("WordTrailsUtilities", "wrap_with_quotes"), $hashes_from_db)) .
		") ORDER BY n.NodeID DESC";
	    $children_from_db = $wpdb->get_results($sql, OBJECT);
	    $countsql = "SELECT COUNT(nr.ChildID) FROM " . WordTrailsGlobal::$tables->node_rel . " as nr WHERE nr.ParentID = %d GROUP BY nr.ParentID";
	    if (is_array($children_from_db)) {
		foreach ($children_from_db as $child) {
		    $alternate = !$alternate;
		    $count++;
		    if ($count <= $start || $count > $start+$maxcount) continue;
		    $numchildren = $wpdb->get_var($wpdb->prepare($countsql, $child->NodeID));
		    $numchildren = (int)$numchildren;?>
    <tr class="<?php echo in_array($child->Hash, $this->flash_change) ?"flash-change":""; echo $alternate ? " alternate":""; ?>">
	<th scope="row" class="check-column"><input type="checkbox" name="process[del_rel][args][children][]" value="<?php echo $child->Hash; ?>" <?php echo ($type == "trail" && $child->Type != "trail") ? "disabled=\"disabled\" " : ""; ?>/></th>
	<td><?php echo $child->Name; ?></td>
	<td><?php echo $child->ShortDesc; ?></td>
	<td class="num"><?php echo $numchildren; ?></td>
	<td class="num"><input type="radio" name="child_radio_<?php echo $child->Hash; ?>" onclick="setDefault('<?php echo $node->getHash(); ?>','<?php echo $child->Hash; ?>', '<?php echo $jumphash; ?>');return false;" <?php echo ($node->isDefault($child->Hash) ? "checked=\"checked\"" : ""); ?> /></td>
	<td class="num">0</td>
	<td>
	    <input id="form_modify_func_<?php echo $child->NodeID; ?>" type="hidden" name="process[modify_<?php echo $child->NodeID; ?>][func]" value="" />
	    <input id="form_modify_arg_hash_<?php echo $child->NodeID; ?>" type="hidden" name="process[modify_<?php echo $child->NodeID; ?>][args][hash]" value="<?php echo $child->Hash; ?>" />
	    <a href="<?php echo $this->href_to_edit_node($child->Hash); ?>" class="ltoh"><img src="<?php echo WordTrailsGlobal::$urlpath; ?>imgs/buttons/edit_l.png" alt="Edit" /></a><?php
	     if ($type == "trail") { ?>

	    <a href="#" class="ltoh" onclick="jQuery('#form_modify_func_<?php echo $child->NodeID; ?>').attr('value','delete_node');jQuery(this).parents('form').submit();"><img src="<?php echo WordTrailsGlobal::$urlpath; ?>imgs/buttons/delete_l.png" alt="Delete" /></a><?php
	     } ?>

	</td>
    </tr><?php
		}
	    }
	}
    } else { ?>

    <tr><td colspan="7" align="center"><i>No <?php echo ucwords($children_type); ?> Found</i></td></tr><?php
    }

    ?>
    </tbody>
</table>
<div class="tablenav">
    <?php if ($page_links) echo "<div class='tablenav-pages'>{$page_links}</div>"; ?>
    <?php if ($type != "trail") { ?><input type="submit" class="<?php wt_button_class(); ?>" onclick="jQuery('#remove_children_func').attr('value', 'delete_relationships');" value="Delete Relationships" /><?php } ?>

</div>
<br class="clear" />
</form><?php
    }

    public function edit_node_add_post_form($hash) {
	global $wpdb, $wp_locale, $wp_query;
	$node = WordTrailsGlobal::getNode($hash);
	$type = $node->getType();
	$jumphash = "#add_post";
	$child_type = "child";
	$children_type = "children";
	if ($type == "trail") {
	    $child_type = "marker";
	    $children_type = "markers";
	}
	$default_ppp = 10;
	$ppp = "posts_per_page";
	if ( !isset( $_GET['paged'] ) )
	    $_GET['paged'] = 1;
	if ( !isset($_GET[$ppp]))
	    $_GET[$ppp] = $default_ppp;
	$vars_to_keep = array($ppp, "s", "cat", "tag", "m", "author", "post_status");
	list($post_stati, $avail_post_stati) = wp_edit_posts_query();
	set_query_var("posts_per_page", $_GET[$ppp]);
	$wp_query->query($wp_query->query_vars);?>

<a name="<?php echo substr($jumphash,1); ?>"></a>
<form id="posts-filter" action="<?php echo $jumphash; ?>" method="get"><?php
foreach (array("page", "mode", "hash") as $var) { ?>

    <input type="hidden" name="<?php echo $var; ?>" value="<?php echo attribute_escape($_GET[$var]); ?>" /><?php
}
?>

<h2 style="float:left;margin:0px;padding:0px;border-bottom: none;"><?php
	$post_status_label = __('All Posts');
	if ( isset($_GET['post_status']) && in_array( $_GET['post_status'], array_keys($post_stati) ) )
        $post_status_label = $post_stati[$_GET['post_status']][1];
	if ( $post_listing_pageable && !is_archive() && !is_search() )
		$h2_noun = is_paged() ? sprintf(__( 'Previous %s' ), $post_status_label) : sprintf(__('Latest %s'), $post_status_label);
	else
		$h2_noun = $post_status_label;
	// Use $_GET instead of is_ since they can override each other
	$h2_author = '';
	$_GET['author'] = (int) $_GET['author'];
	if ( $_GET['author'] != 0 ) {
		if ( $_GET['author'] == '-' . $user_ID ) { // author exclusion
			$h2_author = ' ' . __('by other authors');
		} else {
			$author_user = get_userdata( get_query_var( 'author' ) );
			$h2_author = ' ' . sprintf(__('by %s'), wp_specialchars( $author_user->display_name ));
		}
	} else {
	    unset($_GET['author']);
	}
	$h2_search = isset($_GET['s'])   && $_GET['s']   ? ' ' . sprintf(__('matching &#8220;%s&#8221;'), wp_specialchars( get_search_query() ) ) : '';
	$h2_cat    = isset($_GET['cat']) && $_GET['cat'] ? ' ' . sprintf( __('in &#8220;%s&#8221;'), single_cat_title('', false) ) : '';
	$h2_tag    = isset($_GET['tag']) && $_GET['tag'] ? ' ' . sprintf( __('tagged with &#8220;%s&#8221;'), single_tag_title('', false) ) : '';
	$h2_month  = isset($_GET['m'])   && $_GET['m']   ? ' ' . sprintf( __('during %s'), single_month_title(' ', false) ) : '';
	printf( _c( '%1$s%2$s%3$s%4$s%5$s%6$s|You can reorder these: 1: Posts, 2: by {s}, 3: matching {s}, 4: in {s}, 5: tagged with {s}, 6: during {s}' ), $h2_noun, $h2_author, $h2_search, $h2_cat, $h2_tag, $h2_month );
	?>
</h2>
	<div style="float:right;">
	       <input type="text" id="post-search-input" name="s" value="<?php the_search_query(); ?>" />
	       <input type="submit" value="<?php _e( 'Search Posts' ); ?>" class="<?php wt_alt_button_class(); ?>" />
	</div>
	<br style="clear:both;" />
	<div style="float:left;"><ul class="subsubsub" style="margin:0px;padding:0px;">
	<?php
	$status_links = array();
	$num_posts = wp_count_posts( 'post', 'readable' );
	$class = empty( $_GET['post_status'] ) ? ' class="current"' : '';
	$status_links[] = "<li><a href='" . $this->href_to_edit_node($hash, array_diff($vars_to_keep, array("post_status"))) . "{$jumphash}' $class>" . __('All Posts') . '</a>';
	foreach ( $post_stati as $status => $label ) {
		$class = '';

		if ( !in_array( $status, $avail_post_stati ) )
			continue;

		if ( empty( $num_posts->$status ) )
			continue;
		if ( $status == $_GET['post_status'] )
			$class = ' class="current"';

		$status_links[] = "<li><a href='" . $this->href_to_edit_node($hash, array_diff($vars_to_keep, array("post_status"))) . "&amp;post_status={$status}{$jumphash}' $class>" .
		sprintf( __ngettext( $label[2][0], $label[2][1], $num_posts->$status ), number_format_i18n( $num_posts->$status ) ) . '</a>';
	}
	echo implode( ' |</li>', $status_links ) . '</li>';
	unset( $status_links );
	?>
	</ul></div>
<?php $this->num_per_page($ppp, $default_ppp, $pp, $jumphash); ?>

<?php if ( isset($_GET['post_status'] ) ) : ?>
<input type="hidden" name="post_status" value="<?php echo attribute_escape($_GET['post_status']) ?>" />
<?php endif; ?>
<?php if ( isset($_GET['posts_per_page'] ) ) : ?>
<input type="hidden" name="posts_per_page" value="<?php echo attribute_escape($_GET['posts_per_page']) ?>" />
<?php endif; ?>
<div class="tablenav">

<?php
$page_links = paginate_links( array(
	'base' => add_query_arg( 'paged', '%#%' ) . $jumphash,
	'format' => '',
	'total' => $wp_query->max_num_pages,
	'current' => $_GET['paged']
));

if ( $page_links )
	echo "<div class='tablenav-pages'>$page_links</div>";
?>
<div class="alignleft">
<input type="button" class="<?php wt_button_class(); ?>" name="add_posts" value="Add Posts as <?php echo ucwords($children_type); ?>" onclick="jQuery('#add_posts_form').submit()" />
<?php
$arc_query = "SELECT DISTINCT YEAR(post_date) AS yyear, MONTH(post_date) AS mmonth FROM $wpdb->posts WHERE post_type = 'post' ORDER BY post_date DESC";

$arc_result = $wpdb->get_results( $arc_query );

$month_count = count($arc_result);

if ( $month_count && !( 1 == $month_count && 0 == $arc_result[0]->mmonth ) ) { ?>
<select name='m'>
<option<?php selected( @$_GET['m'], 0 ); ?> value='0'><?php _e('Show all dates'); ?></option>
<?php
foreach ($arc_result as $arc_row) {
	if ( $arc_row->yyear == 0 )
		continue;
	$arc_row->mmonth = zeroise( $arc_row->mmonth, 2 );

	if ( $arc_row->yyear . $arc_row->mmonth == $_GET['m'] )
		$default = ' selected="selected"';
	else
		$default = '';

	echo "<option$default value='$arc_row->yyear$arc_row->mmonth'>";
	echo $wp_locale->get_month($arc_row->mmonth) . " $arc_row->yyear";
	echo "</option>\n";
}
?>
</select>
<?php } ?>

<?php
$dropdown_options = array('show_option_all' => __('View all categories'), 'hide_empty' => 0, 'hierarchical' => 1,
	'show_count' => 0, 'orderby' => 'name', 'selected' => $_GET['cat']);
wp_dropdown_categories($dropdown_options);
do_action('restrict_manage_posts');
?>
<input type="submit" id="post-query-submit" value="<?php _e('Filter'); ?>" class="<?php wt_alt_button_class(); ?>" />
</div>
</div>
</form>

<br class="clear" />
<?php
//-----------Rows Start
?>
<form id="add_posts_form" name="edit_node_add_posts_as_children" method="post" action="<?php echo $this->href_to_submit($hash) . "#edit_children"; ?>">
<input type="hidden" name="process[addpost][func]" value="create_nodes_from_posts" />
<input type="hidden" name="process[addpost][args][parenthash]" value="<?php echo $hash; ?>" />
<table class="widefat">
	<thead>
	<tr>

<?php
$posts_columns = array_diff_key(wp_manage_posts_columns(), array('comments'=>1));
$posts_columns["cb"]= "<input type=\"checkbox\"" . wt_checkAll("document.getElementById('add_posts_form')", true) . " />";

foreach($posts_columns as $post_column_key => $column_display_name) {
	if ( 'cb' === $post_column_key )
		$class = ' class="check-column"';
	elseif ( 'comments' === $post_column_key )
		$class = ' class="num"';
	else
		$class = '';
?>
	<th scope="col"<?php echo $class; ?>><?php echo $column_display_name; ?></th>
<?php } ?>

	</tr>
	</thead>
	<tbody>
<?php
if ( have_posts() ) {
$bgcolor = '';
add_filter('the_title','wp_specialchars');

// Create array of post IDs.
$post_ids = array();
foreach ( $wp_query->posts as $a_post )
	$post_ids[] = $a_post->ID;

$comment_pending_count = get_pending_comments_num($post_ids);
global $post, $id;
while (have_posts()) : the_post();
$class = 'alternate' == $class ? '' : 'alternate';
global $current_user;
$post_owner = ( $current_user->ID == $post->post_author ? 'self' : 'other' );
$title = get_the_title();
if ( empty($title) )
	$title = __('(no title)');
?>
	<tr id='post-<?php echo $id; ?>' class='<?php echo trim( $class . ' author-' . $post_owner . ' status-' . $post->post_status ); ?>' valign="top">

<?php

foreach($posts_columns as $column_name=>$column_display_name) {

	switch($column_name) {

	case 'cb':
		?>
		<th scope="row" class="check-column"><?php if ( current_user_can( 'edit_post', $post->ID ) ) { ?><input type="checkbox" name="process[addpost][args][postids][]" value="<?php the_ID(); ?>" /><?php } ?></th>
		<?php
		break;
	case 'modified':
	case 'date':
		if ( '0000-00-00 00:00:00' == $post->post_date && 'date' == $column_name ) {
			$t_time = $h_time = __('Unpublished');
		} else {
			if ( 'modified' == $column_name ) {
				$t_time = get_the_modified_time(__('Y/m/d g:i:s A'));
				$m_time = $post->post_modified;
				$time = get_post_modified_time('G', true);
			} else {
				$t_time = get_the_time(__('Y/m/d g:i:s A'));
				$m_time = $post->post_date;
				$time = get_post_time('G', true);
			}
			if ( ( abs(time() - $time) ) < 86400 ) {
				if ( ( 'future' == $post->post_status) )
					$h_time = sprintf( __('%s from now'), human_time_diff( $time ) );
				else
					$h_time = sprintf( __('%s ago'), human_time_diff( $time ) );
			} else {
				$h_time = mysql2date(__('Y/m/d'), $m_time);
			}
		}
		?>
		<td><abbr title="<?php echo $t_time ?>"><?php echo apply_filters('post_date_column_time', $h_time, $post, $column_name) ?></abbr></td>
		<?php
		break;
	case 'title':
		?>
		<td><strong><!--a class="row-title" href="<?php echo get_permalink($post->ID); ?>" title="<?php echo attribute_escape(sprintf(__('View "%s"'), $title)); ?>"--><?php echo $title ?></strong>
		<?php if ( !empty($post->post_password) ) { _e(' &#8212; <strong>Protected</strong>'); } elseif ('private' == $post->post_status) { _e(' &#8212; <strong>Private</strong>'); } ?></td>
		<?php
		break;

	case 'categories':
		?>
		<td><?php
		$categories = get_the_category();
		if ( !empty( $categories ) ) {
			$out = array();
			foreach ( $categories as $c )
				$out[] = "<a href='" . $this->href_to_edit_node($hash, array_diff($vars_to_keep, array("tag", "cat"))) . "&amp;cat={$c->term_id}{$jumphash}'> " . wp_specialchars(sanitize_term_field('name', $c->name, $c->term_id, 'category', 'display')) . "</a>";
			echo join( ', ', $out );
		} else {
			_e('Uncategorized');
		}
		?></td>
		<?php
		break;

	case 'tags':
		?>
		<td><?php
		$tags = get_the_tags();
		if ( !empty( $tags ) ) {
			$out = array();
			foreach ( $tags as $c )
				$out[] = "<a href='" . $this->href_to_edit_node($hash, array_diff($vars_to_keep, array("tag", "cat"))) . "&amp;tag={$c->slug}{$jumphash}'> " . wp_specialchars(sanitize_term_field('name', $c->name, $c->term_id, 'post_tag', 'display')) . "</a>";
			echo join( ', ', $out );
		} else {
			_e('No Tags');
		}
		?></td>
		<?php
		break;

	case 'comments':
		?>
		<td class="num"><div class="post-com-count-wrapper">
		<?php
		$left = isset($comment_pending_count) ? $comment_pending_count[$post->ID] : 0;
		$pending_phrase = sprintf( __('%s pending'), number_format( $left ) );
		if ( $left )
			echo '<strong>';
		comments_number("<a href='edit.php?p=$id' title='$pending_phrase' class='post-com-count'><span class='comment-count'>" . __('0') . '</span></a>', "<a href='edit.php?p=$id' title='$pending_phrase' class='post-com-count'><span class='comment-count'>" . __('1') . '</span></a>', "<a href='edit.php?p=$id' title='$pending_phrase' class='post-com-count'><span class='comment-count'>" . __('%') . '</span></a>');
		if ( $left )
			echo '</strong>';
		?>
		</div></td>
		<?php
		break;

	case 'author':
		?>
		<td><a href="<?php echo $this->href_to_edit_node($hash, array_diff($vars_to_keep, array("author"))) . "&amp;author=";the_author_ID();echo $jumphash; ?>"><?php the_author() ?></a></td>
		<?php
		break;

	case 'status':
		?>
		<td>
		<a href="<?php the_permalink(); ?>" title="<?php echo attribute_escape(sprintf(__('View "%s"'), $title)); ?>" rel="permalink">
		<?php
		switch ( $post->post_status ) {
			case 'publish' :
			case 'private' :
				_e('Published');
				break;
			case 'future' :
				_e('Scheduled');
				break;
			case 'pending' :
				_e('Pending Review');
				break;
			case 'draft' :
				_e('Unpublished');
				break;
		}
		?>
		</a>
		</td>
		<?php
		break;

	case 'control_view':
		?>
		<td><a href="<?php the_permalink(); ?>" rel="permalink" class="view"><?php _e('View'); ?></a></td>
		<?php
		break;

	case 'control_edit':
		?>
		<td><?php if ( current_user_can('edit_post',$post->ID) ) { echo "<a href='post.php?action=edit&amp;post=$id' class='edit'>" . __('Edit') . "</a>"; } ?></td>
		<?php
		break;

	case 'control_delete':
		?>
		<td><?php if ( current_user_can('delete_post',$post->ID) ) { echo "<a href='" . wp_nonce_url("post.php?action=delete&amp;post=$id", 'delete-post_' . $post->ID) . "' class='delete'>" . __('Delete') . "</a>"; } ?></td>
		<?php
		break;

	default:
		?>
		<td><?php do_action('manage_posts_custom_column', $column_name, $id); ?></td>
		<?php
		break;
	}
}
?>
	</tr>
<?php
endwhile;
} else {
?>
  <tr style='background-color: <?php echo $bgcolor; ?>'>
    <td colspan="8"><?php _e('No posts found.') ?></td>
  </tr>
<?php
} // end if ( have_posts() )
?>
	</tbody>
</table>
</form>
<?php
//----------Rows End
?>
<div id="ajax-response"></div>

<div class="tablenav">
    <div class="alignleft">
	<input type="button" class="<?php wt_button_class(); ?>" name="add_posts" value="Add Posts as <?php echo ucwords($children_type); ?>" onclick="jQuery('#add_posts_form').submit()" />
    </div>

<?php
if ( $page_links )
	echo "<div class='tablenav-pages'>$page_links</div>";
?>

<br class="clear" />
</div>

<br class="clear" />
	<?php
    }
    public function create_nodes_from_posts($args) {
	extract($args);
	if (!isset($parenthash) || !is_array($postids) || empty($postids)) return false;
	$postids = array_filter($postids, is_numeric);
	foreach ($postids as $pid) {
	    $pid = (int)$pid;
	    $this->new_int_post_node($parenthash, $pid);
	}
	$this->addMessage("Added " . count($postids) . " new Markers from Posts", "markers_to_posts_notice");

	/*
	?><div id="message" class="updated fade">Added <?php echo count($postids); ?> new Markers from Posts</div><?php
	*/
    }

    public function edit_node_add_url_form($hash) {
	$node = WordTrailsGlobal::getNode($hash);
	if (!is_object($node)) return false;
	$this->continueMenu = false;
	$type = $node->getType();
	$child_type = "child";
	$children_type = "children";
	if ($type == "trail") {
	    $child_type = "marker";
	    $children_type = "markers";
	}?>
<form name="edit_node_add_url" method="post" action="<?php echo $this->href_to_submit($hash) . "#edit_children"; ?>">
    <input type="hidden" name="process[url][func]" value="new_ext_url_node" />
    <input type="hidden" name="process[url][args][parenthash]" value="<?php echo $hash; ?>" />
    <label>URL: <input type="text"<?php echo wt_text_box_size(30); ?>name="process[url][args][url]" value="" /></label><br />
    <span class="subsubsub">(This will likely be a dead end on your trail)</span><br /><br />
    <input type="submit" value="Add <?php echo ucwords($child_type); ?>" class="<?php wt_button_class(); ?>" />
</form>
	<?php
    }
    public function edit_node_add_p2p_form($hash) {
	echo "doesn't exist yet";
    }

    public function delete_node($args) {
	extract($args);
	if (!isset($hash)) return false;
	$node = WordTrailsGlobal::getNode($hash);
	if (!is_object($node)) return false;
	if ($node->getType() == "trail") {
	    $node->delete(true);
	    $this->addMessage("1 Trail deleted", "trail_deleted_notice");
	    /*
	    if (!isset($confirm)) {
		array_push($this->messages, "<strong>Are you sure you want to delete this trail and all of its markers?</strong>
    <br /><br /><form name=\"confirm\" method=\"post\" action=\"" . str_replace( '%7E', '~', $_SERVER['REQUEST_URI']) . "\">
    <input type=\"hidden\" name=\"process[confirm][func]\" value=\"delete_node\" />
    <input type=\"hidden\" name=\"process[confirm][args][hash]\" value=\"$hash\" />
    <input type=\"submit\" class=\"". wt_button_class(true) . "\" name=\"process[confirm][args][confirm]\" value=\"Yes - Delete\" />
    <input type=\"submit\" class=\"". wt_button_class(true) . "\" name=\"process[confirm][args][confirm]\" value=\"No - Cancel\" />
    </form>");
		$this->continueMenu = false;
	    } if ((isset($confirm) && stristr($confirm, "yes") !== false)) {
		$node->delete(true);
	    }
	    */
	} else {
	    $node->delete();
	    $this->flash_change[] = $node->getHash();
	}
    }
    public function un_delete_node($args) {
	extract($args);
	if (!isset($hash)) return false;
	$node = WordTrailsGlobal::getNode($hash);
	if (!is_object($node)) return false;
	$node->unFlagForDeletion();
	$this->flash_change[] = $hash;
    }

    public function new_ext_url_node($args) {
	extract($args);
	if (!isset($parenthash) || !isset($url)
	    || empty($parenthash) || empty($url)
	    || !preg_match('/^(http|https|ftp):\/\/[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,6}((:[0-9]{1,5})?\/.*)?$/i' ,$url)) return false;
	$parent = WordTrailsGlobal::getNode($parenthash);
	if (is_object($parent)) {
	    $node = WordTrailsGlobal::createNewNode(false);
	    $node->setURL($url);
	    $node->deriveSite();
	    $node->deriveType();
	    $node->setName($url);
	    if ($parent->getType() == "trail") {
		$node->trail_hash = $parenthash;
	    } else {
		$node->trail_hash = $parent->trail_hash;
		$trail = WordTrailsGlobal::getNode($parent->trail_hash);
		if (is_object($trail))
		    $trail->addChild($node->getHash());
	    }
	    $parent->addChild($node->getHash());
	    $this->flash_change[] = $node->getHash();
	}
    }

    public function new_int_post_node($parenthash, $id) {
	$node = WordTrailsGlobal::createNewNode(true);
	$node->setPostID($id);
	$parent = WordTrailsGlobal::getNode($parenthash);
	if (is_object($parent)) {
	    //this needs revisiting
	    if ($parent->getType() == "trail") {
		$node->trail_hash = $parenthash;
	    } else {
		$node->trail_hash = $parent->trail_hash;
		$trail = WordTrailsGlobal::getNode($parent->trail_hash);
		if (is_object($trail))
		    $trail->addChild($node->getHash());
	    }
	    $parent->addChild($node->getHash());
	    $this->flash_change[] = $node->getHash();
	}
    }

    //---------------------------------------------------------

    public function settingsMenu() {
	//$this->continueMenu = true;
	global $wpdb;
	//$this->processPOST();
	if (!$this->continueMenu) return;
	add_meta_box("wt_settings_trail_page", "Trails Display Page", array($this, "trail_page_form"), "wtsettings");
	add_meta_box("wt_settings_trail_permissions", "Trails Permissions", array($this, "trail_permissions"), "wtsettings");
	add_meta_box("wt_settings_enable_analytics", "Analytics", array($this, "enable_analytics_form"), "wtsettings");
	add_meta_box("wt_trailMeme_registration", "TrailMeme Registration", array($this, "trailMeme_registration_form"), "wtsettings");
	add_meta_box("wt_trail_sync", "Trail Synchronization", array($this, "trail_sync_form"), "wtsettings");
	?>

<div class="wrap">
    <h2 style="border-bottom:none"><?php _e( 'Settings', WordTrailsGlobal::DOMAIN ); ?></h2>
<?php //$this->printMessages(); ?>
<div id="poststuff">

<?php
do_meta_boxes("wtsettings", 'advanced', false);
wp_nonce_field( 'closedpostboxes', 'closedpostboxesnonce', false );
wp_nonce_field( 'meta-box-order', 'meta-box-order-nonce', false );
?>

</div>
</div>
<script language="JavaScript" type="text/javascript">
<!--

    var ajax_url = "<?php bloginfo( 'wpurl' ); ?>/wp-admin/admin-ajax.php";

    <?php post_box_toggles("wtsettings"); ?>

    // close postboxes that should be closed
    jQuery('.if-js-closed').removeClass('if-js-closed').addClass('closed');

    // show things that should be visible, hide what should be hidden
    jQuery('.hide-if-no-js').show();
    jQuery('.hide-if-js').hide();

//-->
</script>
	<?php

    }
    
    public function trail_permissions() {
	global $wp_roles;
	if (empty($wp_roles->roles)) return;?>

	<form id="trail_permissions" method="post" action="<?php echo $this->href_to_submit(); ?>">
	    <input type="hidden" name="process[trail_permissions][func]" value="update_trail_permissions" />
	    <label for="process[trail_permissions][args][role]">This user role and above can save trails:</label>
	    <select name="process[trail_permissions][args][role]">
		<?php
	$editable_roles = get_editable_roles();
	$selected = WordTrailsGlobal::get_option(WordTrailsGlobal::MIN_ROLE_SAVE);
	foreach( $editable_roles as $role => $details ) {
	    $name = translate_user_role($details['name'] );
	    echo "\n\t<option value='" . esc_attr($role) . "'";
	    if ( $selected == $role )
		echo " selected='selected'";
	    echo ">$name</option>";
	}
		?>

	    </select>
	    <input type="submit" class="<?php wt_button_class(); ?>" value="Update" />
	</form><?php
    }
    
    public function update_trail_permissions($args) {
	extract($args);
	WordTrailsGlobal::update_option(WordTrailsGlobal::MIN_ROLE_SAVE, $role);
	WordTrailsUtilities::addCapabilities();
    }

    public function enable_analytics_form() { ?>
    <form id="enable_analytics_form" method="post" action="<?php echo $this->href_to_submit(); ?>">
	<input type="hidden" name="process[enable_analytics][func]" value="update_analytics_settings" />
	<input type="hidden" name="process[enable_analytics][args][ignore]" value="ignore" />
	<label><input type="checkbox" name="process[enable_analytics][args][collect]" <?php echo (WordTrailsGlobal::get_option(WordTrailsGlobal::ANALYTICS_COLLECT_OPTION) == true) ? "checked=\"checked\" " : ""; ?>/>
	    Allow WordTrails to collect annonymous trail following data</label>
	<br />
	<label><input type="checkbox" name="process[enable_analytics][args][transmit]" <?php echo (WordTrailsGlobal::get_option(WordTrailsGlobal::ANALYTICS_TRANSMIT_OPTION) == true) ? "checked=\"checked\" " : ""; echo is_null(WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILMEME_KEY_OPTION)) ? " disabled" : ""; ?>/>
	    Allow automatic analytics reporting to TrailMeme</label>
	<br />
	<input type="submit" class="<?php wt_button_class(); ?>" value="Update" />
    </form><?php
    }

    public function update_analytics_settings($args) {
	extract($args);
	WordTrailsGlobal::update_option(WordTrailsGlobal::ANALYTICS_COLLECT_OPTION, (int)isset($collect));
	WordTrailsGlobal::update_option(WordTrailsGlobal::ANALYTICS_TRANSMIT_OPTION, (int)isset($transmit));
	WordTrailsUtilities::unScheduleAnalytics();
	if ((int)isset($transmit)) WordTrailsUtilities::scheduleAnalytics();

    }
    public function enable_analytics_transmit($args) {
	extract($args);
	error_log("update analytics enabled: " . (int)isset($enable));
	WordTrailsGlobal::update_option(WordTrailsGlobal::ANALYTICS_TRANSMIT_OPTION, (int)isset($enable));
	WordTrailsUtilities::unScheduleAnalytics();
	if ((int)isset($enable)) WordTrailsUtilities::scheduleAnalytics();
    }

    public function trailMemeMenu() {
	//$this->continueMenu = true;
	global $wpdb;
	//$this->processPOST();
	if (!$this->continueMenu) return;


	add_meta_box("wt_trailMeme_registration", "TrailMeme Registration", array($this, "trailMeme_registration_form"), "trailmeme");
	add_meta_box("wt_trail_sync", "Trail Synchronization", array($this, "trail_sync_form"), "trailmeme");
	?>

<div class="wrap">
    <h2 style="border-bottom:none; margin:0px;"><?php _e( 'TrailMeme Settings', WordTrailsGlobal::DOMAIN ); ?></h2>
<?php //$this->printMessages(); ?>
<div id="poststuff">

<?php
do_meta_boxes("trailmeme", 'advanced', false);
wp_nonce_field( 'closedpostboxes', 'closedpostboxesnonce', false );
wp_nonce_field( 'meta-box-order', 'meta-box-order-nonce', false );
?>

</div>
</div>
<script language="JavaScript" type="text/javascript">
<!--

    var ajax_url = "<?php bloginfo( 'wpurl' ); ?>/wp-admin/admin-ajax.php";

    <?php post_box_toggles("trailmeme"); ?>

    // close postboxes that should be closed
    jQuery('.if-js-closed').removeClass('if-js-closed').addClass('closed');

    // show things that should be visible, hide what should be hidden
    jQuery('.hide-if-no-js').show();
    jQuery('.hide-if-js').hide();

//-->
</script><?php
    }

    public function trailMeme_registration_form() {
	if (!WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILMEME_KEY_OPTION)) {
	    ?>

	<h2 style="border-bottom:none; margin:0px;">Register this blog with TrailMeme (http://trailmeme.com/)</h2>
	<form name="TrailMemeRegistration" action="http://trailmeme.com/site_reg/login_first" method="post" target="_blank">
	    <input type="hidden" name="site[hash]" value="<?php echo WordTrailsGlobal::get_option(WordTrailsGlobal::SITE_HASH_OPTION); ?>" />
	    <input type="hidden" name="site[url]" value="<?php echo get_option("home"); ?>" />
	    <input type="hidden" name="site[name]" value="<?php echo get_option("blogname"); ?>" />
	    <input type="hidden" name="site[caption]" value="<?php echo get_option("blogdescription"); ?>" />
	    <input type="hidden" name="version" value="WordTrails  <?php echo WordTrailsGlobal::VERSION; ?>" />
	    <input type="submit" value="Click here to register" class="<?php wt_button_class(); ?>" />
	</form>
	<h2 style="border-bottom:none; margin-bottom:0px;">Enter your TrailMeme Site Key</h2>
	<form name="SaveTrailMemeKey" action="<?php echo $this->href_to_submit(); ?>" method="post">
	    <input type="hidden" name="process[site_key][func]" value="store_site_key" />
	    <label>Site Key: <input type="text"<?php echo wt_text_box_size(20); ?>name="process[site_key][args][key]" /></label>
	    <input type="submit" value="Store" class="<?php wt_button_class(); ?>" />
	</form><?php
	} else {
	    ?>

	<h2 style="border-bottom:none;margin:0px;">You TrailMeme Site Key is stored</h2>
	<form name="delete_site_key" id="delete_site_key" action="<?php echo $this->href_to_submit(); ?>" method="post">
	    <input type="hidden" name="process[delete_key][func]" value="delete_site_key" />
	    <input type="hidden" name="process[delete_key][args][confirm]" value="0" />
	</form>
	<a href="javascript:void(0);" onclick="jQuery('#delete_site_key').submit();">Click here to delete your stored site key</a><br />
	<strong>Warning: This cannot be undone!</strong><?php
	}
    }

    public function trail_sync_form() {
	global $wpdb;
	if (!WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILMEME_KEY_OPTION)) {
	    echo "You must register with TrailMeme before you can synchronize your trails.";
	} else {
	    ?>

	<form name="send_trails" id="send_trails" action="javascript:GatherAndPushTrails('send_trails');" method="get">
	    <strong>Note:</strong> Trail Synchronization is still beta and may not function properly.<br />
	    <strong>Note:</strong> Trail Synchronization only works on trails stored in the databse. If you have any unsaved changes on any trails, they will be <strong>DISCARDED</strong> before upload.
	<div class="tablenav"><input type="submit" class="<?php wt_button_class(); ?>" value="Send Selected Trails" /></div><br class="clear" />
	<table class="widefat">
	    <thead>
		<tr>
		    <th scope="col" class="check-column" width="50"><input type="checkbox"<?php wt_checkAll("document.getElementById('send_trails')"); ?> /></th>
		    <th scope="col" width="300">Trail Name</th>
		    <th scope="col" class="num" width="200">Last Modified</th>
		    <th scope="col" class="num">TrailMeme Status</th>
		</tr>
	    </thead>
	    <tbody><?php
	    $sync_dates = WordTrailsGlobal::getTrailSyncDates();
	    $trails = WordTrailsGlobal::getAllTrailNodes("Name, Modified");
	    $alternate = false;
	    foreach ($trails["db"] as $trail) {
		$alternate = !$alternate; ?>

		<tr id="tr_<?php echo $trail->Hash; ?>"<?php echo $alternate ? " class=\"alternate\"" : ""; ?> >
		    <th scope="row" class="check-column"><input type="checkbox" name="<?php echo $trail->Hash; ?>" /></th>
		    <td><a href="<?php echo $this->href_to_edit_node($trail->Hash); ?>" title="View/Edit Trail"><?php echo $trail->Name; ?></a></td>
		    <td class="num"><?php echo WordTrailsUtilities::stampToAbbr($trail->Modified); ?></td>
		    <td class="num sync_date"><?php echo (isset($sync_dates[$trail->Hash]) ? "Last Sent: " . WordTrailsUtilities::stampToAbbr($sync_dates[$trail->Hash]) : "<i>Never uploaded</i>"); ?></td>
		</tr><?php
	    }
	    foreach ($trails["live"] as $trail) {
		if (!is_numeric($trail->getNodeID())) continue;
		$alternate = !$alternate; ?>

		<tr id="tr_<?php echo $trail->getHash();?>"<?php echo $alternate ? " class=\"alternate\"" : ""; ?>>
		    <th scope="row" class="check-column"><input type="checkbox" name="<?php echo $trail->getHash(); ?>" /></th>
		    <td><a href="<?php echo $this->href_to_edit_node($trail->getHash()); ?>" title="View/Edit Trail"><?php echo $trail->getName(); ?></a><?php
		    if ($trail->hasUnsaved()) {
			echo " - " . count($trail->unsaved()) . " Unsaved Change(s) will be lost";
		    } ?>

		    <td class="num"><?php echo WordTrailsUtilities::stampToAbbr($trail->getModified()); ?></td>
		    <td class="num sync_date"><?php echo (isset($sync_dates[$trail->getHash()]) ? "Last Sent: " . WordTrailsUtilities::stampToAbbr($sync_dates[$trail->getHash()]) : "<i>Never uploaded</i>"); ?></td>
		</tr><?php
	    }

	    ?></tbody>
	</table>
	<div class="tablenav"><input type="submit" class="<?php wt_button_class(); ?>" value="Send Selected Trails" /></div><br class="clear" />
	</form><?php
	}
    }
/* deprecated
    public function trailMeme_print_analytics_form() {
	if (!WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILMEME_KEY_OPTION)) {
	    echo "You must register with TrailMeme before you can send your usage statistics.";
	} else {

	}
    }
*/

    public function trailMeme_usage_analytics_form() {
	if (!WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILMEME_KEY_OPTION)) {
	    echo "You must register with TrailMeme before you can send your usage statistics.";
	} else {
	    echo "TrailMeme is not accepting following analytics at this time.";
	}
    }


    public function store_site_key($args) {
	extract($args);
	if (!isset($key) || empty($key)) return false;
	WordTrailsGlobal::update_option(WordTrailsGlobal::TRAILMEME_KEY_OPTION, $key);
    }

    public function delete_site_key($args) {
	WordTrailsGlobal::update_option(WordTrailsGlobal::TRAILMEME_KEY_OPTION, null);
    }

    public function analyticsMenu() {
	global $wpdb;
	if (isset($_GET["mode"])) {
	    if (false !== array_search($_GET["mode"], array_keys($this->modes))) {
		call_user_func(array($this, $this->modes[$_GET["mode"]]));
	    }
	}
	if (!$this->continueMenu) return;
	$fulldata = WordTrailsAnalytics::pullAnalyticsFor(); ?>

	<div class="wrap">
	    <h2 style="border-bottom:none">Trail Usage &amp; Print Data</h2>
<?php //$this->printMessages(); ?>
	    <div class="alignleft" style="font-size: 12px;">
		<ul class="subsubsub" style="display:inline;">
		    <li><a href="javascript:void(0)" onclick="jQuery('.expand-link').forceShow();">Show All Details</a> |</li>
		    <li><a href="javascript:void(0)" onclick="jQuery('.expand-link').forceHide();">Hide All Details</a></li>
		</ul>
	    </div>
	    <?php $this->displayUsageFor($fulldata); ?>
	</div>
<script language="JavaScript" type="text/javascript">
<!--
    var ajax_url = "<?php bloginfo( 'wpurl' ); ?>/wp-admin/admin-ajax.php";
    // show things that should be visible, hide what should be hidden
    jQuery('.hide-if-no-js').show();
    jQuery('.hide-if-js').hide();
//-->
</script><?php
    }

    public function single_trail_analytics() {
	if (!isset($_GET["trail"]) || empty($_GET["trail"])) return;
	if (!WordTrailsUtilities::isHash($_GET["trail"])) return;
	$fulldata = WordTrailsAnalytics::pullAnalyticsFor($_GET["trail"]); ?>

	<div class="wrap">
	    <h2 style="border-bottom:none">Usage & Print Data for trail: <?php echo WordTrailsGlobal::getNodeNameFromHash($_GET["trail"]); ?></h2>
<?php //$this->printMessages(); ?>
	    <div class="alignleft" style="font-size: 12px;">
		<ul class="subsubsub" style="display:inline;">
		    <li><a href="javascript:void(0)" onclick="jQuery('.expand-link').forceShow();">Show All Details</a> |</li>
		    <li><a href="javascript:void(0)" onclick="jQuery('.expand-row').show().end().find('.expand-link').click();">Hide All Details</a></li>
		</ul>
	    </div>
	    <?php $this->listSIDsForTrail($_GET["trail"], $fulldata); ?>

	    <h4><a href="<?php echo $this->build_href(null, array("mode", "sid", "trail")); ?>">&larr; Back to Full Analytics</a></h4>
	</div>
<script language="JavaScript" type="text/javascript">
<!--
    var ajax_url = "<?php bloginfo( 'wpurl' ); ?>/wp-admin/admin-ajax.php";
    // show things that should be visible, hide what should be hidden
    jQuery('.hide-if-no-js').show();
    jQuery('.hide-if-js').hide();
//-->
</script><?php
	$this->continueMenu = false;
    }

    public function single_sid_analytics() {
	if (!isset($_GET["sid"]) || empty($_GET["sid"])) return;
	$fulldata = WordTrailsAnalytics::pullAnalyticsFor($_GET["sid"], "sid"); ?>

	<div class="wrap">
	    <h2 style="border-bottom:none">Usage & Print Data for session: <?php echo $_GET["sid"]; ?></h2>
	    <?php //$this->displayUsageFor($fulldata, array_keys($fulldata));
		foreach (array_keys($fulldata) as $trail) { ?>

	    <h3><?php echo WordTrailsGlobal::getNodeNameFromHash($trail); ?></h3>
	    <?php
		//$this->printMessages();
		$this->detailedHitAndPrint($trail, $_GET["sid"], $fulldata);

		}
	    ?>

	    <h4><a href="<?php echo $this->build_href(null, array("mode", "sid", "trail")); ?>">&larr; Back to Full Analytics</a></h4>
	</div>
<script language="JavaScript" type="text/javascript">
<!--
    // show things that should be visible, hide what should be hidden
    jQuery('.hide-if-no-js').show();
    jQuery('.hide-if-js').hide();
//-->
</script><?php
	$this->continueMenu = false;
    }

    public function detailedHitAndPrint($trail, $sid, &$fulldata) {
	if (!isset($fulldata[$trail])) return null;
	if (!isset($fulldata[$trail][$sid])) return null;
	if (!isset($fulldata[$trail][$sid]["hit"]) && !isset($fulldata[$trail][$sid]["print"])) return null;
	if (empty($fulldata[$trail][$sid]["hit"]) && empty($fulldata[$trail][$sid]["print"])) return null;
	$data = array_merge((array)$fulldata[$trail][$sid]["hit"], (array)$fulldata[$trail][$sid]["print"]);
	ksort($data, SORT_NUMERIC);

	?>
	<table class="widefat" cellspacing="4">
	    <thead>
	    <tr valign="top" class="thead">
		<th scope="col"><?php _e( 'Time', WordTrailsGlobal::DOMAIN ); ?></th>
		<th scope="col"><?php _e( 'Where', WordTrailsGlobal::DOMAIN ); ?></th>
		<th scope="col"><?php _e( 'How', WordTrailsGlobal::DOMAIN ); ?></th>
	    </tr>
	    </thead>
	    <tbody><?php
	$alternate = false;
	foreach ($data as $stamp => &$hit) { ?>

	    <tr<?php echo $alternate ? " class='alternate'":"";?>>
		<td><?php echo WordTrailsUtilities::stampToAbbr($hit->Stamp); ?></td><?php
	    if (isset($hit->PrintID)) { ?>

		<td colspan="2">Printed From <?php echo WordTrailsAnalytics::$print_buttons[(int)$hit->ButtonID]; ?></td><?php
	    } elseif (isset($hit->UsageID)) {
		if ($hit->Type == WordTrailsAnalytics::WEV && isset($hit->NodeHash))
		    $where = "WEV: " . WordTrailsGlobal::getNodeNameFromHash($hit->NodeHash);
		else
		    $where = "Birds-Eye-View";
		switch ($hit->How) {
		    case WordTrailsAnalytics::INDIRECT:
			$how = "Intersection";
			break;
		    case WordTrailsAnalytics::CHANCE:
			$how = "By Chance";
			break;
		    default:
			$how = "Direct Link";
			break;
		}
		?>

		<td><?php echo $where; ?></td>
		<td><?php echo $how; ?></td><?php
	    } else { ?>

		<td colspan="2" align="center"><i>Unknown Data</i></td><?php
	    } ?>

	    </tr><?php
	    $alternate = !$alternate;
	} ?>

	    </tbody>
	</table><?php
    }

    public function listSIDsForTrail($trail, &$fulldata) {
	global $wp_wt_sort;
	if (!isset($fulldata[$trail]) || empty($fulldata[$trail])) return null;
	$data = array();
	foreach ($fulldata[$trail] as $sid => &$usage) {
	    $data[$sid] = array();
	    if (isset($usage["hit"]) && !empty($usage["hit"])) {
		$data[$sid]["hit_count"] = count($usage["hit"]);
		$first = reset($usage["hit"]);
		$last = end($usage["hit"]);
		$data[$sid]["first"] = !empty($first->NodeHash) ? WordTrailsGlobal::getNodeNameFromHash($first->NodeHash) : "Birds-Eye-View";
		$data[$sid]["last"] = !empty($last->NodeHash) ? WordTrailsGlobal::getNodeNameFromHash($last->NodeHash) : "Birds-Eye-View";
	    } else {
		$data[$sid]["hit_count"] = 0;
		//this is only possible if printed from trails listing
	    }
	    if (isset($usage["print"]) && !empty($usage["print"])) {
		$data[$sid]["print"] = count($usage["print"]);
	    } else {
		$data[$sid]["print"] = 0;
	    }
	}
	$sort_before = $wp_wt_sort;
	if (isset($_GET["wt_sid_sort"]) && !empty($_GET["wt_sid_sort"]))
	    $wp_wt_sort = $_GET["wt_sid_sort"];
	else $wp_wt_sort = "sid";
	$reverse = false;
	if (stristr($wp_wt_sort, "_R")) {
	    $reverse = true;
	    $wp_wt_sort = substr($wp_wt_sort, 0, -2);
	}
	if ($wp_wt_sort == "sid")
	    ksort($data);
	else
	    uasort($data, array("WordTrailsUtilities", "array_multi_sort"));
	if ($reverse)
	    $data = array_reverse($data);

	$spp = "sessions_per_page";
	$sp = "sessions_paged";
	$total = count($data);
	$maxcount = (int)$_GET[$spp]>0 ? (int)$_GET[$spp]:10;
	$page = (int)$_GET[$sp]>0 ? (int)$_GET[$sp]:1;
	$start = ($page-1)*$maxcount;
	while ($start > $total) {
	    $start -= $maxcount;
	    $page--;
	}
	$page_links = paginate_links( array(
	    'base' => add_query_arg( $sp, '%#%' ),
	    'format' => '',
	    'total' => ceil($total/$maxcount),
	    'current' => $page
	));
	$this->num_per_page($spp, 10, $sp);
	$data = array_slice($data, $start, $maxcount);
	if ($page_links) { ?>
	<div class="tablenav">
	    <div class='tablenav-pages'><?php echo $page_links; ?></div>
	</div>
	<br class="clear" /><?php
	} ?>

	<table class="widefat" cellspacing="4">
	    <thead>
	    <tr valign="top" class="thead">
		<th scope="col" class="num"><a href="<?php echo $this->build_href(array("wt_sid_sort"=>"sid".($wp_wt_sort == "sid" && !$reverse ? "_R" : "")));?>"><?php _e( 'SessionID', WordTrailsGlobal::DOMAIN ); ?></a></th>
		<th scope="col" class="num"><a href="<?php echo $this->build_href(array("wt_sid_sort"=>"hit_count".($wp_wt_sort == "hit_count" && !$reverse ? "_R" : "")));?>"><?php _e( '# of Hits', WordTrailsGlobal::DOMAIN ); ?></a></th>
		<th scope="col"><a href="<?php echo $this->build_href(array("wt_sid_sort"=>"first".($wp_wt_sort == "first" && !$reverse ? "_R" : "")));?>"><?php _e( 'Entry Point', WordTrailsGlobal::DOMAIN ); ?></a></th>
		<th scope="col"><a href="<?php echo $this->build_href(array("wt_sid_sort"=>"last".($wp_wt_sort == "last" && !$reverse ? "_R" : "")));?>"><?php _e( 'Last Visited', WordTrailsGlobal::DOMAIN ); ?></a></th>
		<th scope="col"><a href="<?php echo $this->build_href(array("wt_sid_sort"=>"print".($wp_wt_sort == "print" && !$reverse ? "_R" : "")));?>"><?php _e( 'Print?', WordTrailsGlobal::DOMAIN ); ?></a></th>
		<th scope="col"><?php _e( 'Details', WordTrailsGlobal::DOMAIN ); ?></th>
	    </tr>
	    </thead>
	    <tbody><?php
	$alternate = true;
	$expanded = (array)WordTrailsGlobal::get_option(WordTrailsGlobal::ANALYTICS_EXPANDED_OPTION);
	foreach ($data as $sid => $info) {
	    $alternate = !$alternate; ?>

	    <tr<?php echo $alternate ? " class='alternate'" : ""; ?>>
		<th scope="row" class="num"><a href="<?php echo $this->build_href(array("mode"=>"sid_analytics", "sid"=>$sid)); ?>"><?php echo $sid; ?></a></th>
		<td class="num"><?php echo $info["hit_count"]; ?> Hits</td>
		<td><?php echo $info["first"]; ?></td>
		<td><?php echo $info["last"]; ?></td>
		<td><?php echo $info["print"] > 0 ? "Yes ({$info["print"]})" : "No"; ?></td>
		<td><a href="javascript:void(0);" class="expand-link">Show/Hide Details</a></td>
	    </tr>
	    <tr id="single_sid_stats_<?php echo "{$sid}_{$trail}"; ?>" class="expand-row<?php echo !in_array("single_sid_stats_{$sid}_{$trail}", $expanded) ? " hide-if-js" : "";?>">
		<td colspan="6" align="left" valign="top">
		    <?php $this->detailedHitAndPrint($trail, $sid, $fulldata); ?>

		</td>
	    </tr><?php
	}
	$wp_wt_sort = $sort_before; ?>

	    </tbody>
	</table><?php
	if ($page_links) { ?>
	<div class="tablenav">
	    <div class='tablenav-pages'><?php echo $page_links; ?></div>
	</div>
	<br class="clear" /><?php
	}
    }

    public function displayUsageFor(&$fulldata, $trails = array()) {
	global $wp_wt_sort;
	if (empty($trails)) $trails = WordTrailsGlobal::getAllTrailHashes();
	$data = array();
	foreach ($trails as $trail) {
	    $data[$trail] = array();
	    $data[$trail]["name"] = WordTrailsGlobal::getNodeNameFromHash($trail);
	    if (isset($fulldata[$trail]) && !empty($fulldata[$trail])) {
		$data[$trail]["follows"] = count($fulldata[$trail]);
		$data[$trail]["print"] = 0;
		$data[$trail]["depth"] = array();
		foreach ($fulldata[$trail] as $sid => $session) {
		    if (isset($session["print"]) && !empty($session["print"]))
			$data[$trail]["print"] += count($session["print"]);
		    if (isset($session["hit"]) && !empty($session["hit"]))
			$data[$trail]["depth"][] = count($session["hit"]);
		}
		$data[$trail]["depth"] = round(array_sum($data[$trail]["depth"])/count($data[$trail]["depth"]), 2);
	    } else
		$data[$trail]["empty"] = true;
	}
	$sort_before = $wp_wt_sort;
	if (isset($_GET["wt_trail_sort"]) && !empty($_GET["wt_trail_sort"]))
	    $wp_wt_sort = $_GET["wt_trail_sort"];
	else $wp_wt_sort = "name";
	$reverse = false;
	if (stristr($wp_wt_sort, "_R")) {
	    $reverse = true;
	    $wp_wt_sort = substr($wp_wt_sort, 0, -2);
	}
	uasort($data, array("WordTrailsUtilities", "array_multi_sort"));
	if ($reverse)
	    $data = array_reverse($data);

	$tpp = "trails_per_page";
	$tp = "trails_paged";
	$total = count($data);
	$maxcount = (int)$_GET[$tpp]>0 ? (int)$_GET[$tpp]:10;
	$page = (int)$_GET[$tp]>0 ? (int)$_GET[$tp]:1;
	$start = ($page-1)*$maxcount;
	while ($start > $total) {
	    $start -= $maxcount;
	    $page--;
	}
	$page_links = paginate_links( array(
	    'base' => add_query_arg( $tp, '%#%' ),
	    'format' => '',
	    'total' => ceil($total/$maxcount),
	    'current' => $page
	));
	$this->num_per_page($tpp, 10, $tp);
	$data = array_slice($data, $start, $maxcount);
	if ($page_links) { ?>
	<div class="tablenav">
	    <div class='tablenav-pages'><?php echo $page_links; ?></div>
	</div>
	<br class="clear" /><?php
	} ?>

	<table class="widefat" cellspacing="4">
	    <thead>
	    <tr valign="top" class="thead">
		<th scope="col"><a href="<?php echo $this->build_href(array("wt_trail_sort"=>"name".($wp_wt_sort == "name" && !$reverse ? "_R" : "")));?>"><?php _e( 'Trail Name', WordTrailsGlobal::DOMAIN ); ?></a></th>
		<th scope="col" class="num"><a href="<?php echo $this->build_href(array("wt_trail_sort"=>"follows".($wp_wt_sort == "follows" && !$reverse ? "_R" : "")));?>"><?php _e( '# of Unique Follows', WordTrailsGlobal::DOMAIN ); ?></a></th>
		<th scope="col" class="num"><a href="<?php echo $this->build_href(array("wt_trail_sort"=>"depth".($wp_wt_sort == "depth" && !$reverse ? "_R" : "")));?>"><?php _e( 'Average Read Depth', WordTrailsGlobal::DOMAIN ); ?></a></th>
		<th scope="col" class="num"><a href="<?php echo $this->build_href(array("wt_trail_sort"=>"print".($wp_wt_sort == "print" && !$reverse ? "_R" : "")));?>"><?php _e( '# of Prints', WordTrailsGlobal::DOMAIN ); ?></a></th>
		<th scope="col"><?php _e( 'Details', WordTrailsGlobal::DOMAIN ); ?></th>
	    </tr>
	    </thead>
	    <tbody><?php
	$alternate = true;
	$expanded = (array)WordTrailsGlobal::get_option(WordTrailsGlobal::ANALYTICS_EXPANDED_OPTION);
	foreach ($data as $hash => $info) {
	    $alternate = !$alternate; ?>

	    <tr<?php echo $alternate==true ? " class='alternate'" : ""; ?>><?php
	    if ($info["empty"] == true) { ?>

		<th scope="row"><?php echo $info["name"]; ?></th>
		<td class="num"><i>No Usage Or Print Data</i></td>
		<td colspan="3"></td><?php
		continue;
	    } ?>

		<th scope="row"><a href="<?php echo $this->build_href(array("mode"=>"trail_analytics", "trail"=>$hash)); ?>"><?php echo $info["name"]; ?></a></th>
		<td class="num"><?php echo $info["follows"]; ?></td>
		<td class="num"><?php echo $info["depth"]; ?></td>
		<td class="num"><?php echo $info["print"]; ?></td>
		<td><a href="javascript:void(0);" class="expand-link">Show/Hide Details</a></td>
	    </tr>
	    <tr id="single_trail_stats_<?php echo $hash; ?>" class="expand-row<?php echo !in_array("single_trail_stats_{$hash}", $expanded) ? " hide-if-js" : "";?>">
		<td colspan="5">
		    <?php $this->listSIDsForTrail($hash, $fulldata); ?>

		</td>
	    </tr><?php
	}
	$wp_wt_sort = $sort_before; ?>

	    </tbody>
	</table><?php
	if ($page_links) { ?>
	<div class="tablenav">
	    <div class='tablenav-pages'><?php echo $page_links; ?></div>
	</div>
	<br class="clear" /><?php
	}
    }

    public function trail_page_form() {
	global $wpdb;

	$pid = WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILS_PAGE_OPTION);
	$title = null;
	$query = array();
	if (is_numeric($pid)) $query = query_posts("page_id=" . $pid);
	if (empty($query)) {
	    $pid = null;
	    WordTrailsGlobal::update_option(WordTrailsGlobal::TRAILS_PAGE_OPTION, null);
	}
	if (is_numeric($pid)) {
	    $title = get_the_title((int)$pid);
	    ?>

    <form name="remove_current_trails_page" id="remove_current_trails_page" action="<?php echo $this->href_to_submit(); ?>" method="post">
    <input type="hidden" name="process[remove_current][func]" value="" id="remove_current" />
    <input type="hidden" name="process[remove_current][args][confirm]" value="false" />
    <h2 style="border-bottom:none; margin:0px;">Current Display Page: <strong><em><?php echo $title; ?></em></strong></h2>
    <input type="button" value="Disassociate With Page" class="<?php wt_button_class(); ?>"  onclick="jQuery('#remove_current').attr('value', 'disassociate_current_trails_page');jQuery('#remove_current_trails_page').submit();" />
    <input type="button" value="Delete Page" class="<?php wt_button_class(); ?>" onclick="jQuery('#remove_current').attr('value', 'delete_current_trails_page');jQuery('#remove_current_trails_page').submit();" />
    </form><?php
	} ?>
    <h2 style="border-bottom:none; margin-bottom:0px;">New Trails Display Page</h2>
    <form name="new_trails_page" id="new_trails_page" action="<?php echo $this->href_to_submit(); ?>" method="post">
    <input type="hidden" name="process[new_trails_page][func]" value="new_trails_page" />
    <label>Page Name or ID: <input type="text" name="process[new_trails_page][args][input]" id="new_trails_page_name" value="Trails" onkeyup="checkNewTrailsPage();" /></label>
    <input type="submit" class="<?php wt_button_class(); ?>" id="new_trails_submit" value="Make New Trails Page" disabled="disabled" /><br />
    <small>Enter the name or ID of an existing page, or enter the name for a new page.</small>
    </form><?php
    }

    public function new_trails_page($args) {
	extract($args);
	if (is_null($input)) return false;
	if (is_numeric($input))
	    $query = query_posts("page_id=" . $input);
	else
	    $query = query_posts("pagename=" . $input);
	if (!empty($query)) {
	    $post_data = array();
	    $post_data["post_ID"] = $query[0]->ID;
	    $post_data["post_type"] = "page";
	    $post_data['comment_status'] = 'closed';
	    $post_data['ping_status'] = 'closed';
	    edit_post($post_data);
	    WordTrailsGlobal::update_option(WordTrailsGlobal::TRAILS_PAGE_OPTION, $query[0]->ID);
	} else {
	    $_POST['post_date'] = date("Y-m-d H:i:s");
	    $_POST['post_date_gmt'] = get_gmt_from_date( $_POST['post_date'] );
	    $_POST['post_type'] = "page";
	    $_POST['post_status'] = 'publish';
	    $_POST['comment_status'] = 'closed';
	    $_POST['ping_status'] = 'closed';
	    $_POST['post_title'] = $input;
	    $id = write_post();
	    if (is_numeric($id))
		WordTrailsGlobal::update_option(WordTrailsGlobal::TRAILS_PAGE_OPTION, $id);
	}
    }

    public function disassociate_current_trails_page($args) {
	$pid = WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILS_PAGE_OPTION);
	WordTrailsGlobal::update_option(WordTrailsGlobal::TRAILS_PAGE_OPTION, null);
	if (is_numeric($pid))
	    return $pid;
    }

    public function delete_current_trails_page($args) {
	extract($args);
	if ($confirm != "false") {
	    if (stristr($confirm, "yes")) {
		$this->continueMenu = false;
		$pid = $this->disassociate_current_trails_page(true);
	        //delete it too
		//echo "<div class='message'>" . clean_url(wp_nonce_url(get_option("home") . "/wp-admin/post.php?action=delete&post=" . $pid, "delete")) . "</div>";
		if (is_numeric($pid))
		    header("Location: " . get_option("home") . "/wp-admin/page.php?action=delete&post=" . $pid . "&_wpnonce=" . wp_create_nonce('delete-page_' . $pid));
	    }
	} else {
	    $title = "Are you sure you want to delete the entire page? This process cannot be undone.";
	    $msg = "<form name=\"delete_confirm\" action=\"" . $this->href_to_submit() . "\" method=\"post\">
	<input type=\"hidden\" name=\"process[delete_trails_page][func]\" value=\"delete_current_trails_page\" />
	<input type=\"submit\" class=\"{$wt_button_class}\" name=\"process[delete_trails_page][args][confirm]\" value=\"Yes - Delete the entire page\" />
	<input type=\"submit\" class=\"{$wt_button_class}\" name=\"process[delete_trails_page][args][confirm]\" value=\"No\" />
	</form>";
	$this->addMessage($msg, "delete_page_confirm_notice", $title);
	$this->continueMenu = false;
	}
    }

}
?>