<?php
require_once "WordTrailsGlobal.inc";

class WordTrailsUtilities {
    public static $statics_set = false;
    public static $tables_set = false;

    public static function reqClasses() {
        foreach (WordTrailsGlobal::$node_classes as $where => $types) {
            foreach ($types as $type => $class) {
                require_once $class.".inc";
            }
        }
	require_once "WordTrailsWidget.inc";
        require_once "WordTrailsTagManager.inc";
        require_once "WordTrailsMenu.inc";
        require_once "WordTrailsAnalytics.inc";
	require_once "WordTrailsControlManager.inc";
    }

    //---------OPTIONS----------
    public static function option_exists( $setting ) {
        $alloptions = wp_load_alloptions();
	if ( isset( $alloptions[$setting] ) ) return true;
        if ( false !== wp_cache_get( $setting, 'options' ) ) return true;
        return false;
    }

    public static function initialize_option ( $setting, $default ) {
        if (self::option_exists($setting)) return get_option($setting);
        add_option($setting, $default);
        return $default;
    }
    
    public static function updateNewOptions() {
	if (!self::option_exists(WordTrailsGlobal::ALL_OPTIONS)) WordTrailsGlobal::get_all_options();
	else {
	    $options = get_option(WordTrailsGlobal::ALL_OPTIONS);
	    foreach($options as $key => $val) {
		WordTrailsGlobal::$options[$key] = $val;
	    }
	    update_option(WordTrailsGlobal::ALL_OPTIONS, WordTrailsGlobal::$options);
	}
	$site_hash = WordTrailsGlobal::get_option(WordTrailsGlobal::SITE_HASH_OPTION);
	if (is_null($site_hash) || empty($site_hash))
	    WordTrailsGlobal::update_option(WordTrailsGlobal::SITE_HASH_OPTION, self::newHash());
    }
    //-------ENDOPTIONS---------

    //-----------MISC-----------
    public static function newHash() {
        global $wpdb;
        return $wpdb->get_var("SELECT UUID()");
    }

    public static function isHash($hash) {
        $expl_hash = explode("-", $hash);
        if (count($expl_hash) == 5 && strlen(implode("",$expl_hash)) == 32)
            return true;
        return false;
    }

    public function stampToAbbr($mysqlTimestamp) {
        $time = strtotime($mysqlTimestamp);
        $long = date(__('Y/m/d g:i:s A'), $time);
        $h_time = date(__('Y/m/d'), $time);
        if (abs(time() - $time) < 86400) {
            if (time() - $time < 0)
                $h_time = sprintf( __('%s from now'), human_time_diff( $time ) );
            else
		$h_time = sprintf( __('%s ago'), human_time_diff( $time ) );
        }
        return "<abbr title=\"". $long ."\">" . $h_time . "</abbr>";
    }

    public static function wrap_with_quotes($a) {
	return "\"" . $a . "\"";
    }
    
    public static function plain_quotes($string) {
	$search = array(chr(145), chr(146), chr(147), chr(148), chr(151));
	$replace = array("'", "'", '"', '"', '-');
	$string = str_replace($search, $replace, $string);
	$search = array('&lsquo;', '&rsquo;', '&ldquo;', '&rdquo;', '&mdash;');
	$string = str_replace($search, $replace, $string);
	$search = array("" . chr(152), "" . chr(153), '' . chr(156), '' . chr(157),  '"');
	return str_replace($search, $replace, $string);
    } 

    public static function is_serialized_trail($data) {
	if (!is_serialized($data)) return false;
	foreach (WordTrailsGlobal::$node_classes as $where => $types) {
	    if (strstr($data, $types["trail"]))
		return true;
	}
	return false;
    }

    public static function filter_unsaved_only($n) {
	if (is_serialized($n)) return true;
	if (is_object($n)) {
	    if ($n->hasUnsaved()) return true;
	    return false;
	}
	$node = WordTrailsGlobal::getNode($n, false);
	if (!is_object($node)) return false;
	if ($node->hasUnsaved()) return true;
	return false;
    }


    public static function sort_roles($a, $b) {
	if ($a["numcaps"] == $b["numcaps"]) {
	    return 0;
	}
	return ($a["numcaps"] > $b["numcaps"]) ? -1 : 1;
    }


    public static function is_newer_version($override = true) {
        $installed = split(".", WordTrailsGlobal::get_option(WordTrailsGlobal::VERSION_OPTION));
        $mine = split(".", WordTrailsGlobal::VERSION);
        for ($i = 0; $i < 3; $i++) {
            if ($installed[$i] == $mine[$i]) continue;
            if ($installed[$i] < $mine[$i]) {
                if ($override) WordTrailsGlobal::update_option(WordTrailsGlobal::VERSION_OPTION, WordTrailsGlobal::VERSION);
                return true;
            }
            if ($installed[$i] > $mine[$i]) return false;
        }
        return false;
    }


    public static function setTables() {
	if (self::$tables_set) return false;
        global $wpdb;
	WordTrailsGlobal::$tables = (object)array(
            "node" => $wpdb->prefix . "wt_node",
            "node_rel" => $wpdb->prefix . "wt_node_relationship",
            "node_tag" => $wpdb->prefix . "wt_node_tag",
            "tag" => $wpdb->prefix . "wt_tag",
            "external" => $wpdb->prefix . "wt_external",
            "push" => $wpdb->prefix . "wt_trail_push",
            "usage" => $wpdb->prefix . "wt_usage_analytics",
            "usage_push" => $wpdb->prefix . "wt_usage_push",
            "print" => $wpdb->prefix . "wt_print_analytics",
            "print_push" => $wpdb->prefix . "wt_print_push",
	    "control" => $wpdb->prefix . "wt_control",
	    "rel_control" => $wpdb->prefix . "wt_relationship_control"
        );
	self::$tables_set = true;
	return true;
    }
    
    public static function pushDefaultControls() {
	global $wpdb;
	$sql = "REPLACE INTO " . WordTrailsGlobal::$tables->control . " (`ControlID`, `Term`, `Name`, `Image`, `isForward`, `Implies`) VALUES
	(1, 'auto', '(Automatic)', 'straight.png', 1, 2),
	(2, 'auto', '(Automatic)', 'straight_back.png', 0, 1),
	(3, 'straight', 'Straight', 'straight.png', 1, 4),
	(4, 'straight_back', 'Straight', 'straight_back.png', 0, 3),
	(5, 'curve_up', 'Curve Up', 'curve_up.png', 1, 6),
	(6, 'curve_down_back', 'Curve Down', 'curve_down_back.png', 0, 5),
	(7, 'curve_down', 'Curve Down', 'curve_down.png', 1, 8),
	(8, 'curve_up_back', 'Curve Up', 'curve_up_back.png', 0, 7);";
	$wpdb->query($sql);
    }
    public static function addCapabilities() {
	require_once trailingslashit(ABSPATH . WPINC) ."capabilities.php";
	global $wp_roles;
	$role_caps = array();
	foreach ($wp_roles->roles as $role_name => $role_array) {
	    $role_caps[] = array("key" => $role_name, "name" => $role_array["name"], "numcaps" => count($role_array["capabilities"]));
	}
	usort($role_caps, array("WordTrailsUtilities", "sort_roles"));
	$min_role = strtolower(WordTrailsGlobal::get_option(WordTrailsGlobal::MIN_ROLE_SAVE));
	$add = true;
	for ($i=0; $i<count($role_caps); $i++) {
	    if ($add) $wp_roles->add_cap($role_caps[$i]["key"], WordTrailsGlobal::CAP_SAVE);
	    else $wp_roles->remove_cap($role_caps[$i]["key"], WordTrailsGlobal::CAP_SAVE);
	    if (strtolower($role_caps[$i]["key"]) == $min_role || strtolower($role_caps[$i]["name"]) == $min_role) $add = false;
	}
    }
    
    public static function delCapabilities() {
	require_once trailingslashit(ABSPATH . WPINC) ."capabilities.php";
	global $wp_roles;
	foreach ($wp_roles->roles as $role_name => $role_array) {
	    $wp_roles->remove_cap($role_name, WordTrailsGlobal::CAP_NEW);
	    $wp_roles->remove_cap($role_name, WordTrailsGlobal::CAP_MOD);
	    $wp_roles->remove_cap($role_name, WordTrailsGlobal::CAP_SAVE);
	}
    }
    

    public static function setStatics() {
        self::setTables();
	if (self::$statics_set) return false;
	load_plugin_textdomain(WordTrailsGlobal::DOMAIN, trailingslashit(WP_PLUGIN_DIR).dirname(dirname(plugin_basename(__FILE__))));
        if (session_id() == "") session_start();

	WordTrailsGlobal::$abspath = trailingslashit(WP_PLUGIN_DIR) . dirname(dirname(plugin_basename(__FILE__))) . "/";
	WordTrailsGlobal::$urlpath = trailingslashit(WP_PLUGIN_URL) . dirname(dirname(plugin_basename(__FILE__))) . "/";

        self::reqClasses();

	self::$statics_set = true;
	return true;
    }

    public static function makeWidget($pid, $args) {
        $widg = new WordTrailsWidget($pid, $args);
        $widg->displayWidget();
    }
    
    /* for future version
    public static function &makePrintNode($trailHash) {
	$trail = WordTrailsGlobal::getNode($trailHash);
	if (is_object($trail)) {
	    $print = $trail->getPrintNode();
	    if (is_object($print)) return $print;
	}
	$print = WordTrailsGlobal::createNewNode(true, "print");
	$print->fromTrail($trailHash);
	return $print;
    } */

    public static function installMenus() {
	WordTrailsGlobal::makeInstance();
	WordTrailsGlobal::$instance->menu->installMenus();
	global $wp_roles;
    	//superprint($wp_roles->roles, true);
    }
    public static function trailsMenu() {
	WordTrailsGlobal::$instance->menu->trailsMenu();
    }
    public static function add_new_trail() {
	WordTrailsGlobal::$instance->menu->add_new_trail();
    }
    public static function settingsMenu() {
	WordTrailsGlobal::$instance->menu->settingsMenu();
    }
    public static function trailMemeMenu() {
	WordTrailsGlobal::$instance->menu->trailMemeMenu();
    }
    public static function analyticsMenu() {
	WordTrailsGlobal::$instance->menu->analyticsMenu();
    }
    public static function menu_head() {
	WordTrailsGlobal::$instance->menu->insertCSS();
    }
    public static function warnTrailPage() {
	return is_null(WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILS_PAGE_OPTION));
    }
    public static function warnWidget() {
	$widget = false;
	$sidebars = get_option('sidebars_widgets');
	foreach ($sidebars as $bar) {
	    if (in_array('wordtrails-worms-eye-view', (array)$bar)) {
		$widget = true;
		break;
	    }
	}
	return !$widget;
    }
    public static function warnTrailMeme() {
	return is_null(WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILMEME_KEY_OPTION));
    }

    public static function scheduleAnalytics() {
	error_log("schedule hook: " . WordTrailsGlobal::ANALYTICS_TRANSMIT_OPTION);
	wp_schedule_event(time()+5, 'weekly', WordTrailsGlobal::ANALYTICS_TRANSMIT_OPTION);
	error_log("next scheduled: " . wp_next_scheduled(WordTrailsGlobal::ANALYTICS_TRANSMIT_OPTION));
	error_log("now: " . time());
    }

    public static function unScheduleAnalytics() {
	error_log("clear hook: " . WordTrailsGlobal::ANALYTICS_TRANSMIT_OPTION);
	wp_clear_scheduled_hook(WordTrailsGlobal::ANALYTICS_TRANSMIT_OPTION);
    }
    //---------ENDMISC----------

    //----------URLS-----------
    public static function permalink($pid = null, $tid = null) {
        if (is_null($pid)) {
            $pid = (int)WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILS_PAGE_OPTION);
        } else {
            $pid = (int)$pid;
        }
        $url = get_permalink($pid);
        if ($url === false) {
            $url = get_option('home');
            return $url;
        }
        if (!is_null($tid)) {
            if (false === strpos($url, "?")) {
                $url .= "?";
            } else {
                $url .= "&";
            }
            $url .= 't=' . $tid;
        }
        return $url;
    }

    public static function appendTrail($url, $trail_hash_or_id) {
        $tid = WordTrailsGlobal::getNodeIDFromHash($trail_hash_or_id);
        if (false === strpos($url, "?")) {
            $url .= "?";
        } else {
            if (false !== strpos($url, "?t=") || false !== strpos($url, "&t=")) return $url;
            $url .= "&";
        }
        $url .= 't=' . $tid;
        return $url;
    }

    public static function redirect_link_to($qstr, $val = null) {
        $url = get_option('home') . "/";
        if (in_array($qstr, array_keys(WordTrailsGlobal::$redirect_query_strings))) {
            $url .= "?" . WordTrailsGlobal::$redirect_query_strings[$qstr] . "=";
            if (!is_null($val))
                $url .= $val;
        }
        return $url;
    }
    //--------ENDURLS----------

    //-----------XML-----------
    public static function createBasicXMLNode($name, $value = null, &$dom = null) {
        if (is_null($dom))
            $dom = new DOMDocument("1.0");
        $node = $dom->createElement($name);
        if (!is_null($value) && (!empty($value) || $value === 0)) {
            $node->appendChild($dom->createTextNode($value));
        }
        return $node;
    }

    public static function explodeToXMLNodes($bigname, $littlename = "tag", $haystack, $split = null, &$dom = null) {
        if (is_null($dom))
            $dom = new DOMDocument("1.0");
        $node = $dom->createElement($bigname);
        if (is_string($haystack)) {
            if (!is_null($split)) {
                $haystack = explode($split, $haystack);
                $haystack = array_map(trim, $haystack);
            } else {
                $haystack = array();
            }
        }
        if (empty($littlename)) $littlename = "tag";
        if (!is_array($haystack)) echo "not";
        foreach ($haystack as $needle) {
            if (!empty($needle)) {
                $temp = $dom->createElement($littlename);
                $temp->appendChild($dom->createTextNode($needle));
                $node->appendChild($temp);
            }
        }
        return $node;
    }

    public static function extractXMLNode($node, &$store = null) {
        if ($node->hasChildNodes()) {
            $val = null;
            if (count($node->childNodes)== 1 && $node->firstChild->nodeType == XML_TEXT_NODE) {
                $val = $node->firstChild->nodeValue;
                    //echo "extracted: $val from $node->nodeName\n";
                if (is_numeric($val)) {
                    $val = round($val, 2);
                }
            } else if (count($node->childNodes) > 0) {
                $doc = new DOMDocument("1.0");
                $doc->formatOutput = false;
                $doc->appendChild($doc->importNode($node,true));
                //error_log("extract: " . $doc->saveXML());
                foreach ($node->childNodes as $child) {
                    $val[] = self::extractXMLNode($child);
                }
            }
            if (!is_null($val)) {
                if (!is_null($store))
                    $store = $val;
                return $val;
            }
        }
        return null;
    }

    public static function prepareForXML(&$dom = null) {
        if (is_null($dom)) $dom = new DOMDocument("1.0");
        $root = $dom->createElement("traildoc");
        $root->setAttribute("application", "WordPress");
        $root->setAttribute("version", WordTrailsGlobal::XMLVERSION);
        $dom->appendChild($root);
        return array($dom, $root);
    }
    //---------ENDXML----------

    //--------SORTING----------
    public static function sort_hash($a, $b, $first = null) {
        if (!is_null($first) && $a == $first) return -1;
        if (!is_null($first) && $b == $first) return 1;
        if ($a == $b) return 0;
        return strcasecmp($a, $b);
    }

    public static function sort_trails($a, $b) {
        $na = WordTrailsGlobal::getNode($a, false);
        $nb = WordTrailsGlobal::getNode($b, false);
        if (is_object($na) && !is_object($nb)) return -1;
        if (is_object($nb) && !is_object($na)) return 1;
        $first = null;
        WordTrailsGlobal::makeInstance();
        if (!is_null(WordTrailsGlobal::$instance->current_trail))
            $first = WordTrailsGlobal::$instance->current_trail;
        return self::sort_hash($a, $b, $first);
    }

    public static function sort_default_children($a, $b) {
        if ($a["default"]) return -1;
        if ($b["default"]) return 1;
        return 0;
    }

    public static function array_multi_sort($a, $b) {
	global $wp_wt_sort;
	if (!isset($a[$wp_wt_sort]) && !isset($b[$wp_wt_sort])) return 0;
	if (!isset($a[$wp_wt_sort])) return -1;
	if (!isset($b[$wp_wt_sort])) return 1;
	if (is_numeric($a[$wp_wt_sort])) {
	    if ($a[$wp_wt_sort] < $b[$wp_wt_sort]) return -1;
	    if ($a[$wp_wt_sort] > $b[$wp_wt_sort]) return 1;
	    return 0;
	} else return strcasecmp($a[$wp_wt_sort], $b[$wp_wt_sort]);
    }

    public static function array_object_sort($a, $b) {
	global $wp_wt_sort;
	if (!isset($a->{$wp_wt_sort}) && !isset($b->{$wp_wt_sort})) return 0;
	if (!isset($a->{$wp_wt_sort})) return -1;
	if (!isset($b->{$wp_wt_sort})) return 1;
	if (is_numeric($a->{$wp_wt_sort})) {
	    if ($a->{$wp_wt_sort} < $b->{$wp_wt_sort}) return -1;
	    if ($a->{$wp_wt_sort} > $b->{$wp_wt_sort}) return 1;
	    return 0;
	} else return strcasecmp($a->{$wp_wt_sort}, $b->{$wp_wt_sort});
    }
    
    public static function onlyWEV($a) {
	if ($a->type == "BEV" || empty($a->NodeHash)) return false;
	return true;
    }
    
    public static function extractNodeHash($a) {
	return $a->NodeHash;
    }
    //------ENDSORTING---------
}

function superprint($mixed, $pre = false) {
    if ($pre) echo "<pre>";
    print_r($mixed);
    if ($pre) echo "</pre>";
}
function superdump($mixed, $pre = false) {
    if ($pre) echo "<pre>";
    var_dump($mixed);
    if ($pre) echo "</pre>";
}
?>