<?php
/*
Plugin Name: WordTrails
Plugin URI: http://open.xerox.com/xeroxproject/xerox-trails
Description: WordTrails (beta) allows for the creation of multi-branched trails of navigation through WordPress Blogs
Author: Jesse Silverstein (Xerox Corporation)
Version: 0.9.0c
Author URI: http://blog.trailmeme.com/
*/

/*  Copyright 2008-2009  Xerox Corporation  (email : jesse.silverstein@xerox.com)

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

*/

/**
 * Note: For compactness, the TrailViewer source files (.fla and .as) are not
 * included in this package. They can be downloaded from
 * <http://open.xerox.com/xeroxproject/xerox-trails>
 */

/**
 * This is the initial file for the WordTrails plugin for WordPress.
 *
 * This file contains all of the initialization functions for WordTrails. It
 * adds all of the action and filter hooks necesary.
 *
 * @package WordTrails
 * @since 0.0.1
 * @author Jesse Silverstein <jesse.silverstein@xerox.com>
 * @copyright 2008-2009 XIG: SemPrint
 * @global WordPress $wpdb WordPress database management class
 */

require_once "includes/WordTrailsGlobal.inc";

// {{{ wp_wt_install()

/**
 * Registers the WordTrails plugin with WordPress
 *
 * @global WordPress $wpdb WordPress database management class
 * @global string $wp_wt_version Current Version of this plugin
 */
function wp_wt_install() {
    global $wpdb;
    WordTrailsUtilities::updateNewOptions();
    WordTrailsGlobal::destroyFirst();

    require_once "includes/dbstruct.inc";
    
    WordTrailsUtilities::scheduleAnalytics();
    WordTrailsUtilities::pushDefaultControls();
    WordTrailsUtilities::addCapabilities();
}

function wp_wt_deactivate() {
    WordTrailsUtilities::unScheduleAnalytics();
    WordTrailsUtilities::delCapabilities();
}

// }}}
register_activation_hook(__FILE__,'wp_wt_install');
register_deactivation_hook(__FILE__, 'wp_wt_deactivate');

function wp_wt_add_weekly_schedule($schedules = array()) {
    $weekly = array('weekly' => array( 'interval' => 604800, 'display' => __('Once Weekly')));
    return array_merge($schedules, $weekly);
}

function wp_wt_edit_form_advanced() {
    global $post;
    add_meta_box("trail_blazing", "WordTrails", array("WordTrailsGlobal", "editFormBox"), "post", "advanced");
}

function wp_wt_widget_init() {
    register_sidebar_widget('WordTrails Worms-Eye-View', 'wp_wt_widget_worms_eye_view');
}
function wp_wt_widget_worms_eye_view($args) {
    global $wp_the_query;
    if ((is_single() || is_page()) && !is_page(WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILS_PAGE_OPTION))) {
        WordTrailsUtilities::makeWidget($wp_the_query->post->ID,$args);
    }
}

function wp_wt_template_redirect($args) {
    if (is_null(WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILS_PAGE_OPTION))) return;
    if (is_page(WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILS_PAGE_OPTION))) {
	//include("trail.php");
	//exit;
	include "includes/hijack.inc";
    }
}

function wp_wt_head() {
    WordTrailsUtilities::setStatics();
    wp_register_script("ltoh", WordTrailsGlobal::$urlpath . "js/ltoh.js", array("jquery"));
    $sidebars = get_option('sidebars_widgets');
    foreach ($sidebars as $bar) {
	if (in_array('wordtrails-worms-eye-view', (array)$bar)) {
	    include_once "includes/tooltip.inc";
	    wp_enqueue_script("WordTrailsWidget", WordTrailsGlobal::$urlpath . "js/widget.js", array("ltoh"));
	    if (function_exists("wp_enqueue_style")) {
		wp_enqueue_style("WordTrailsWidget", WordTrailsGlobal::$urlpath . "css/widget.css");
	    } else {
		add_action("wp_head", "wp_wt_widget_css");
	    }
	    break;
	}
    }
    if (is_page(WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILS_PAGE_OPTION))) {
	if (!empty($_GET["print"])) {
	    wp_enqueue_script("jquery-ui-draggable");
	    wp_enqueue_script("jquery-ui-sortable");
	}
	include_once "includes/tooltip.inc";
	wp_enqueue_script("ltoh");
	//wp_enqueue_script("WordTrailsWidget", WordTrailsGlobal::$urlpath . "js/widget.js", array("jquery", "tooltip"));
	if (function_exists("wp_enqueue_style")) {
	    wp_enqueue_style("WordTrailsIndex", WordTrailsGlobal::$urlpath . "css/trailindex.css");
	} else {
	    add_action("wp_head", "wp_wt_index_css");
	}
    }
}

function wp_wt_widget_css() {
    WordTrailsUtilities::setStatics();?>

<link rel="stylesheet" type="text/css" href="<?php echo WordTrailsGlobal::$urlpath; ?>css/widget.css" /><?php
}

function wp_wt_index_css() {
    WordTrailsUtilities::setStatics();?>

<link rel="stylesheet" type="text/css" href="<?php echo WordTrailsGlobal::$urlpath; ?>css/trailindex.css" /><?php
}

function wp_wt_track_hits() {
    global $wp_the_query;
    if (is_single()||is_page()) WordTrailsGlobal::trackHits($wp_the_query->post->ID);
}

function wp_wt_admin_init() {
    WordTrailsUtilities::setStatics();
    if (DOING_AJAX)
	require_once "includes/ajax.inc";
}

add_action('admin_menu', array("WordTrailsUtilities", "installMenus"));
add_action('shutdown', array("WordTrailsGlobal", "sessionStore"));
add_action('get_header', 'wp_wt_head');
add_action('loop_start', 'wp_wt_track_hits');
add_action('template_redirect', 'wp_wt_template_redirect');
add_action('parse_request', array("WordTrailsGlobal", "capture_redirect"));
add_action('edit_form_advanced', 'wp_wt_edit_form_advanced');
add_action('save_post', array("WordTrailsGlobal", "blazeTrail"));
add_action('delete_post', array("WordTrailsGlobal", "deletePID"));
add_action('widgets_init', 'wp_wt_widget_init');
add_filter('cron_schedules', 'wp_wt_add_weekly_schedule');
add_action(WordTrailsGlobal::ANALYTICS_TRANSMIT_OPTION, array("WordTrailsGlobal", "sendAnalyticsToTrailMeme"));
add_action('set_current_user', array("WordTrailsUtilities", "setStatics"));
add_action("admin_init", "wp_wt_admin_init");
?>